<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

class pkg_rstroneInstallerScript
{
	public function preflight($type, $parent)
	{
		$app = JFactory::getApplication();

		// check Joomla version compatibility
		$jversion = new JVersion();
		if (!$jversion->isCompatible('3.2.0'))
		{
			$app->enqueueMessage('Please upgrade to at least Joomla! 3.2.0 before continuing!', 'error');

			return false;
		}

		// disable RSTemplate! system plugin
		$pluginExtension = JTable::getInstance('Extension', 'JTable');
		$pluginExtension->load(array('name' => 'plg_system_rstemplate', 'type' => 'plugin'));
		
		if (!empty($pluginExtension->enabled) && !empty($pluginExtension->extension_id))
		{
			$app = JFactory::getApplication();
			$db		= JFactory::getDbo();
			$query	= $db->getQuery(true);
			$query->update($db->qn('#__extensions'))
				  ->set($db->qn('enabled').' = 0')
				  ->where($db->qn('extension_id').' = '.$pluginExtension->extension_id);
			$db->setQuery($query);
			$db->execute();
			
			$app->enqueueMessage('The installer has found and disabled RSTemplate! system plugin which is no longer needed.', 'notice');
		} else {
			return true;
		}
	}

	public function postflight($type, $parent)
	{
		if ($type == 'uninstall')
		{
			return true;
		}

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		if (!class_exists('RSTroneCompiler') && is_file(JPATH_PLUGINS . '/system/rstrone/classes/rscompiler.php'))
		{
			require_once JPATH_PLUGINS . '/system/rstrone/classes/rscompiler.php';
		}

		if ($type == 'update' && class_exists('RSTroneCompiler'))
		{
			$app = JFactory::getApplication();

			$query->clear();
			$query->select($db->qn(array('id', 'params')))
				->from($db->qn('#__template_styles'))
				->where($db->qn('template') . ' = ' . $db->q('rstrone'));
			$db->setQuery($query);
			$templateStyles = $db->loadObjectList();
			foreach ($templateStyles as $style)
			{
				$params = json_decode($style->params);

				$values = array(
					// Colors
					'primary_color'        => $params->{'primary_color'},
					'secondary_color'      => $params->{'secondary_color'},
					'body_bg_color'        => $params->{'body_bg_color'},
					'text_color'           => $params->{'text_color'},
					'heading_color'        => $params->{'heading_color'},
					'link_color'           => $params->{'link_color'},
					'footer_bg_color'      => $params->{'footer_bg_color'},
					'footer_text_color'    => $params->{'footer_text_color'},
					'navbar_bg_color'      => $params->{'navbar_bg_color'},
					'navbar_text_color'    => $params->{'navbar_text_color'},

					// Font
					'googleTitleFont'      => $params->{'googleTitleFont'},
					'googleContentFont'    => $params->{'googleContentFont'},
					'googleNavbarFont'     => $params->{'googleNavbarFont'},
					'font_size_base'       => $params->{'font_size_base'},
					'font_size_navbar'     => $params->{'font_size_navbar'},

					// Visual
					'border_radius'        => $params->{'border_radius'},
					'button_border_radius' => $params->{'button_border_radius'}
				);

				$data = array('params' => $values);
				ob_start();
				$compiler = new RSTroneCompiler();
				$compiler->compile((int) $style->id, $data);
				$message = ob_get_contents();
				$message = json_decode($message);

				ob_end_clean();

				if (!$message->status)
				{
					$app->enqueueMessage(JText::_('TPL_RSTEMPLATE_LESS_COMPILE_ERROR') . '<br/>' . $message->message, 'warning');
				}
			}
			
			// Remove permissions from users with incorrect sample data
			$query = $db->getQuery(true)
				->select($db->qn('id'))
				->from($db->qn('#__users'))
				->where($db->qn('email') . ' LIKE ' . $db->q('%@rsjoomla.com'));
			if ($user_ids = $db->setQuery($query)->loadColumn())
			{
				foreach ($user_ids as $user_id)
				{
					$juser = JUser::getInstance($user_id);
					if (in_array(8, $juser->groups))
					{
						// Anonymize data
						list($user, $domain) = explode('@', $juser->email, 2);
						$juser->email = $user . strtolower(JUserHelper::genRandomPassword(8)) . '@' . strtolower(JUserHelper::genRandomPassword(8)) . $domain;
						
						$juser->save(true);
						
						// Set as Registered
						JUserHelper::setUserGroups($user_id, array(2));
					}
				}
			}
		}
		?>
		<style>
			/* isis backend template fix */
			@media (min-width : 768px) and (max-width : 1120px) {
				.row-fluid [class*="span"] {
					margin-left : 10px;
				}
			}

			@media (max-width : 767px) {
				#system-message-container.j-toggle-main, #j-main-container.j-toggle-main, #system-debug.j-toggle-main {
					float : none;
				}
			}

			/* End isis backend template fix */
			#j-main-container > .span12 > strong {
				font-weight : normal;
			}

			.template-intro {
				margin-bottom : 30px;
			}

			.template-intro *,
			.template-intro *::before,
			.template-intro *::after {
				box-sizing : border-box;
			}

			.template-intro-container {
				margin    : 0 auto;
				max-width : 85%;
			}

			.template-intro-container .btn {
				font-size     : 90%;
				font-weight   : bold;
				margin-bottom : 10px;
			}

			.intro-header {
				text-align : center;
				padding    : 60px 60px 0;
				background : url('../templates/rstrone/images/installer/bg.jpg') no-repeat top left;
			}

			.intro-header-title {
				font-weight   : normal;
				color         : rgb(255, 255, 255);
				margin-bottom : 60px;
			}

			.intro-header-title small {
				color : rgba(255, 255, 255, 0.5);
			}

			.intro-header-screens {
				position : relative;
				bottom   : -8px;
			}

			.intro-footer, .intro-content-highlights, .intro-content-features {
				padding : 60px 20px;
			}

			.intro-content-highlights {
				background-color : #e2e0da;
			}

			.intro-quote {
				max-width  : 70%;
				text-align : center;
				font-style : italic;
				opacity    : 0.75;
				margin     : 0 auto 50px auto;
			}

			.intro-section-title {
				font-weight    : normal;
				text-align     : center;
				padding-bottom : 20px;
				margin-bottom  : 40px;
				border-bottom  : 1px solid rgba(0, 0, 0, 0.15);
				font-size      : 40px;
				line-height    : 1;
			}

			.thumbnails .row-fluid .span4 {
				margin-bottom : 40px;
			}

			.thumbnails .row-fluid .span12 {
				margin-bottom : 40px;
				width         : 97%;
			}

			.thumbnail {
				padding          : 30px;
				border           : 1px solid rgba(0, 0, 0, 0.15);
				background-color : rgba(255, 255, 255, 0.35);
				box-shadow       : none;
			}

			.thumbnail h4 {
				line-height : 1.3;
				font-size   : 18px;
				margin      : 25px 0 0 0;
				text-align  : center;
			}

			.intro-feature .btn,
			.thumbnail .btn {
				margin-top : 20px;
			}

			.intro-feature h4 {
				margin-bottom : 25px;
			}

			.intro-feature + .intro-feature {
				margin-top : 40px;
			}

			.intro-footer {
				text-align : center;
				background : #202935;
				color      : #ffffff;
			}

			.version-history {
				margin          : 0 auto 2rem auto;
				padding         : 0;
				list-style-type : none;
				color           : #ffffff;
			}

			.version-history > li {
				margin      : 0 0 0.5em 0;
				padding     : 0 0 0 4em;
				text-align  : left;
				font-weight : normal;
			}

			.version-new,
			.version-fixed,
			.version-upgraded {
				float                 : left;
				font-size             : 0.8em;
				margin-left           : -4.9em;
				width                 : 4.5em;
				color                 : white;
				text-align            : center;
				font-weight           : bold;
				text-transform        : uppercase;
				-webkit-border-radius : 4px;
				-moz-border-radius    : 4px;
				border-radius         : 4px;
			}

			.version-new {
				background : #7dc35b;
			}

			.version-fixed {
				background : #e9a130;
			}

			.version-upgraded {
				background : #61b3de;
			}

			.install-ok {
				background : #7dc35b;
				color      : #ffffff;
				padding    : 3px;
			}

			.install-not-ok {
				background : #E9452F;
				color      : #ffffff;
				padding    : 3px;
			}

			#installer-left {
				border : 1px solid #e0e0e0;
				float  : left;
				margin : 10px;
			}

			#installer-right {
				float : left;
			}

			.tpl-button {
				display               : inline-block;
				background            : #459300 url(../templates/rstrone/images/bg-button-green.gif) top left repeat-x !important;
				border                : 1px solid #459300 !important;
				padding               : 2px;
				color                 : #fff !important;
				cursor                : pointer;
				margin                : 0;
				-webkit-border-radius : 5px;
				-moz-border-radius    : 5px;
				border-radius         : 5px;
				text-decoration       : none !important;
			}

			.tpl-button:hover {
				text-decoration : underline !important;
			}

			.big-warning {
				background : #FAF0DB;
				border     : solid 1px #EBC46F;
				padding    : 5px;
			}

			.big-warning b {
				color : red;
			}

		</style>
		<div class="template-intro">
			<div class="intro-header">
				<div class="template-intro-container">
					<div class="row-fluid">
						<h1 class="intro-header-title">RSTrone!
							<small>Multipurpose Joomla 3.X Template</small>
						</h1>
					</div>
					<div class="row-fluid">
						<img class="intro-header-screens" src="../templates/rstrone/images/installer/devices.png" alt="">
					</div>
				</div>
			</div>
			<!-- .intro-header -->
			<div class="intro-content">
				<section class="intro-content-highlights">
					<div class="template-intro-container">
						<div class="row-fluid">
							<p class="intro-quote lead">RSTrone! is a fully responsive, highly customizable template compatible with Joomla! 3.x. Its flexibility in customization is given by its multitude of configuration parameters and the flawless compatibility with both RSPageBuilder! and K2 content building tools.</p>
						</div>
						<div class="row-fluid">
							<div class="span12">
								<h3 class="intro-section-title"><strong>RSTrone!</strong> Highlights</h3>
							</div>
						</div>
						<div class="thumbnails">
							<div class="row-fluid">
								<div class="span12">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/13-front-end-customizer.jpg" alt="">
										<h4>Front end customizer</h4>
									</div>
								</div>
							</div>
							<div class="row-fluid">
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/1-12-color-schemes.jpg" alt="">
										<h4>12 color schemes</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/2-clean-design.jpg" alt="">
										<h4>Clean design</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/3-responsive-layout.jpg" alt="">
										<h4>Mobile friendly layout</h4>
									</div>
								</div>
							</div>
							<div class="row-fluid">
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/4-preloader-effects.jpg" alt="">
										<h4>Preloader effects</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/5-google-fonts.jpg" alt="">
										<h4>Google fonts</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/6-login-and-search.jpg" alt="">
										<h4>Login and search menu items</h4>
									</div>
								</div>
							</div>
							<div class="row-fluid">
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/7-social-integrations.jpg" alt="">
										<h4>Social integrations</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/8-instant-copyright.jpg" alt="">
										<h4>Instant copyright</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/9-element-animations.gif" alt="">
										<h4>Element animations</h4>
									</div>
								</div>
							</div>
							<div class="row-fluid">
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/10-template-positions.jpg" alt="">
										<h4>Over 40 module positions</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/11-k2-compatibility.jpg" alt="">
										<h4>Seamless K2 compatibility</h4>
									</div>
								</div>
								<div class="span4">
									<div class="thumbnail">
										<img src="../templates/rstrone/images/installer/12-create-content-with-rspagebuilder.jpg" alt="">
										<h4>RSPageBuilder! elements styling</h4>
									</div>
								</div>
							</div>
						</div>
					</div>
				</section>
				<!-- .intro-content-highlights -->
			</div>
			<!-- .intro-content -->
			<div class="intro-footer">
				<h3 class="text-center">RSTrone! v1.0.5 Changelog</h3>
				<ul class="version-history">
					<li><span class="version-fixed">Fix</span> Sample data was not being installed in Joomla! 3.9.0</li>
				</ul>
				<div class="template-intro-container">
					<a class="btn btn-success btn-large" href="index.php?option=com_templates">Start using RSTrone!</a>
					<a class="btn btn-info btn-large" href="https://www.rsjoomla.com/support/documentation/rstrone.html">Read the RSTrone! User Guide</a>
					<a class="btn btn-warning btn-large" href="http://www.rsjoomla.com/customer-support/tickets.html">Get Support!</a>
				</div>
			</div>
			<!-- .intro-footer -->
		</div>
		<?php
	}
}
