/*
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

;
(function ($, window, document, undefined) {

	// Create the defaults once
	var pluginName = "customizer",
		defaults = {
			template_id: null
		};

	// The actual plugin constructor
	function Plugin(element, options) {
		this.element = element;
		this.options = $.extend({}, defaults, options);

		this._defaults = defaults;
		this._name = pluginName;

		this.init();
	}

	Plugin.prototype = {
		init        : function () {
			options = this.options;
			// Editable LESS variables
			var editable = [
				// Colors
				'primary_color',
				'secondary_color',
				'body_bg_color',
				'text_color',
				'heading_color',
				'link_color',
				'footer_bg_color',
				'footer_text_color',
				'navbar_bg_color',
				'navbar_text_color',
				// Font
				'googleTitleFont',
				'googleContentFont',
				'googleNavbarFont',
				'font_size_base',
				'font_size_navbar',
				// Visual
				'border_radius',
				'button_border_radius'
			];

			var context = '#rstpl-customizer-form';
			var url = rs_root + '/index.php?rstpl_task=compile';

			/**
			 * Open the customizer, and store its state in the localStorage
			 * If we navigate to another page, it will remain open/minimized/closed
			 */
			jQuery('.toggleCustomizer').click(function () {
				jQuery(this).parents('.customizer').toggleClass('open');
				if (typeof(Storage) !== "undefined") {
					localStorage['customizer'] = jQuery(context).parent().attr('class');
				}
			});

			if (typeof(Storage) !== "undefined") {
				this.checkStorage(context);
			}

			this.closeSession();
			this.compileLess(context, url, editable);
		},
		/**
		 * Check the state of the customizer (from the localStorage)
		 * @param context
		 */
		checkStorage: function (context) {
			if (localStorage['customizer'] == 'customizer') {
				jQuery(context).parent().removeClass('open');
			}
		},

		/**
		 * Clears the localStorage when X button is clicked
		 */
		closeSession: function () {
			function removeParam(key, sourceURL) {
				var rtn = sourceURL.split("?")[0],
					param,
					params_arr = [],
					queryString = (sourceURL.indexOf("?") !== -1) ? sourceURL.split("?")[1] : "";
				if (queryString !== "") {
					params_arr = queryString.split("&");
					for (var i = params_arr.length - 1; i >= 0; i -= 1) {
						param = params_arr[i].split("=")[0];
						if (param === key) {
							params_arr.splice(i, 1);
						}
					}
					rtn = rtn + "?" + params_arr.join("&");
				}
				return rtn;
			}

			jQuery('.closeCustomizer').click(function (context) {
				if (jQuery(this).hasClass('requested')) {
					return false;
				}
				
				jQuery.ajax({
					type   : "POST",
					url    : rs_root + '/index.php?rstpl_task=close',
					success: function (result) {
						jQuery(document).find('.customizer').hide('slow').remove();
						if (typeof(Storage) !== "undefined") {
							localStorage.removeItem(['customizer']);
						}
						var url = removeParam("rstpl_task", window.location.href);
						window.location.replace(url);
					}
				});
			});
		},
		compileLess : function (context, url, editable) {
			jQuery('.compileLess').click(function () {
				if (jQuery(this).hasClass('requested')) {
					return false;
				}
				
				var pageWrapper = '#rstpl-pagewrap';

				/**
				 * When we start compiling the less files, we prepend an overlay with a loading icon
				 */
				jQuery(pageWrapper).prepend('<div class="tpl-overlay"></div>');
				jQuery(pageWrapper).append('<div class="sk-folding-cube"><div class="sk-cube1 sk-cube"></div><div class="sk-cube2 sk-cube"></div><div class="sk-cube4 sk-cube"></div><div class="sk-cube3 sk-cube"></div></div>');

				/**
				 * Start creating the object
				 * @type {{compileCss: string}}
				 */
				var data = {'compileCss': 'true'},
					action = jQuery(this).attr('data-action'),
					variables = jQuery(context).find('input, select'),
					theme,
					jsonURL,
					themeJSON;

				switch (action) {
				/**
				 * on save, we build the DATA object from the input fields
				 */
					case 'save':
						variables.each(function () {
							if (this.value != '') {
								var self = this;
								var classes = jQuery(this).attr('class').toString().split(' ');

								jQuery.each(getMatch(classes, editable), function () {
									data[this] = self.value;
								});

							}
						});
						break;

				/**
				 * If it's a predefined theme (chosen from the button), load it from the JSON
				 */
					case 'predefinedTheme':
						// Get the theme
						theme = jQuery(this).attr('data-customizer');
						// JSON Request
						jsonURL = rs_base_path + '/css/themes/themes.json';
						themeJSON = getJSON(jsonURL);
						data = themeJSON[theme];
						break;

				/**
				 * By default, we load the 'Color Mix 1' Setup
				 */
					default:
						theme = 'Color Mix 1';
						jsonURL = rs_base_path + '/css/themes/themes.json';
						themeJSON = getJSON(jsonURL);
						data = themeJSON[theme];
						break;
				}

				/**
				 * Make the AJAX call
				 */
				saveData(url, data);
				
			});

			/**
			 * Function to get matching elements of array
			 * @param a
			 * @param b
			 * @returns {Array}
			 */
			function getMatch(a, b) {
				var matches = [];

				for (var i = 0; i < a.length; i++) {
					for (var e = 0; e < b.length; e++) {
						if (a[i] === b[e]) matches.push(a[i]);
					}
				}

				return matches;
			}

			// Function to save data
			function saveData(url, data) {

				jQuery.ajax({
					type   : "POST",
					url    : url,
					data   : data,
					success: function (result) {
						/**
						 *  we reload the page after the call was successful
						 */
						result = JSON.parse(result);
						var pageWrapper = '#rstpl-pagewrap';

						/**
						 * If the result returns true, we reload the page
						 */
						if (result.status) {
							location.reload();
						}

						/**
						 * else, we remove the preloader and prepend the message to
						 * the system container
						 */
						if (typeof(result.message) !== 'undefined') {
							jQuery('.tpl-overlay, .sk-folding-cube').remove();
							jQuery(pageWrapper).prepend('<div class="container"><div class="alert"><button type="button" class="close" data-dismiss="alert">&times;</button>' + result.message + '</div></div>');
						}
					}
				});
			}

			// Function to get the JSON
			function getJSON(url) {
				var xhReq = new XMLHttpRequest();
				xhReq.open("GET", url, false);
				xhReq.send(null);
				var themeJSON = xhReq.response;

				return jQuery.parseJSON(themeJSON);
			}
		}
	};

	// Plugin wrapper around the constructor,
	// preventing against multiple instantiations
	$.fn[pluginName] = function (options) {
		return this.each(function () {
			if (!$.data(this, "plugin_" + pluginName)) {
				$.data(this, "plugin_" + pluginName,
					new Plugin(this, options));
			}
		});
	};
	
	// Prevent multiple AJAX requests in the same time
	jQuery(document).ajaxStart(function() {
		jQuery(document).find('.customizer .customize-buttons a').each(function() {
			jQuery(this).addClass('requested');
		});
	});
	
	jQuery(document).ajaxStop(function() {
		jQuery(document).find('.customizer .customize-buttons a').each(function() {
			jQuery(this).removeClass('requested');
		});
	});
	
})(jQuery, window, document);
