<?php
/**
* @package RSSeo!
* @copyright (C) 2017 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined( '_JEXEC' ) or die( 'Restricted access' );

class plgRsseoK2 extends JPlugin
{	
	public function rsseo_structuredTabs() {
		JForm::addFormPath(dirname(__FILE__));
		
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		$form	= JForm::getInstance('rsseok2', 'rsseok2', array('control' => 'jform', 'load_data' => true));
		$data	= array();
		
		$query->select($db->qn('data'))
			->from($db->qn('#__rsseo_data'))
			->where($db->qn('type').' = '.$db->q('k2'));
		$db->setQuery($query);
		if ($config = $db->loadResult()) {
			try {
				$registry = new JRegistry;
				$registry->loadString($config);
				$data['k2'] = $registry->toArray();
			} catch (Exception $e) {
				$data['k2'] = array();
			}
		}
		
		$form->bind($data);
		
		echo JHtml::_('bootstrap.addTab', 'structuredData', 'k2', JText::_('K2'));
		foreach ($form->getFieldset('k2') as $field) {
			echo $field->renderField();
		}
		echo JHtml::_('bootstrap.endTab');
	}
	
	public function rsseo_generate_k2($args) {
		$data = $args['data'];
		
		if ($this->getValue($data, 'enable')) {
			$input	= JFactory::getApplication()->input;
			$option	= $input->get('option');
			$view	= $input->get('view');
			$id		= $input->getInt('id');
			$json	= array();
			$array	= array();
			
			if ($option == 'com_k2' && $view == 'item' && $id) {
				if ($article = $this->getArticle($id)) {
					$text	 = substr(strip_tags($article->introtext.$article->fulltext), 0, 500);
					$lang	 = $article->language === '*' ? JFactory::getConfig()->get('language') : $article->language;
					$pName = $this->getValue($data, 'name');
					$pLogo = $this->getValue($data, 'logo');
					$image = JPATH_SITE.'/media/k2/items/src/'.md5('Image'.$article->id).'.jpg';
					$modified = $article->modified && $article->modified != '0000-00-00 00:00:00' ? $article->modified : $article->publish_up;
					
					$array['@context'] = 'https://schema.org';
					$array['@type'] = 'Article';
					$array['mainEntityOfPage']['@type'] = 'WebPage';
					$array['mainEntityOfPage']['@id'] = JUri::root();
					$array['headline'] = $article->title;
					$array['description'] = $text;
					$array['inLanguage'] = $lang;
					$array['interactionCount'] = $article->hits;
					$array['datePublished'] = JHtml::_('date', $article->publish_up, 'c');
					$array['dateCreated'] = JHtml::_('date', $article->created, 'c');
					$array['dateModified'] = JHtml::_('date', $modified, 'c');
					
					if (file_exists($image)) {
						$array['image']['@type'] = 'ImageObject';
						$array['image']['url'] = JUri::root().'/media/k2/items/src/'.md5('Image'.$article->id).'.jpg';
						$array['image']['height'] = '800';
						$array['image']['width'] = '800';
					}
					
					if ($article->name) {
						$array['author']['@type'] = 'Person';
						$array['author']['name'] = $article->name;
					}
					
					if ($article->rating && $article->rating_count) {
						$array['aggregateRating']['@type'] = 'AggregateRating';
						$array['aggregateRating']['ratingValue'] = $article->rating;
						$array['aggregateRating']['reviewCount'] = $article->rating_count;
					}
					
					if ($pName && $pLogo) {
						$array['publisher']['@type'] = 'Organization';
						$array['publisher']['name'] = $pName;
						$array['publisher']['logo']['@type'] = 'ImageObject';
						$array['publisher']['logo']['url'] = JUri::root().$pLogo;
						$array['publisher']['logo']['width'] = '600';
						$array['publisher']['logo']['height'] = '600';
					}
					
					$json[] = '<script type="application/ld+json">';
					$json[] = json_encode($array, $this->json_options());
					$json[] = '</script>';
					
					$args['json'][] = implode("\n",$json);
				}
			}
		}
	}
	
	protected function getArticle($id) {
		$db		= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$query->select($db->qn('a.id'))->select($db->qn('a.title'))
			->select($db->qn('a.introtext'))->select($db->qn('a.fulltext'))
			->select($db->qn('a.publish_up'))->select($db->qn('a.modified'))
			->select($db->qn('a.language'))->select($db->qn('u.name'))
			->select($db->qn('a.created'))->select($db->qn('a.hits'))
			->select('ROUND(v.rating_sum / v.rating_count, 0) AS rating, v.rating_count as rating_count')
			->from($db->qn('#__k2_items','a'))
			->join('LEFT', $db->qn('#__users','u').' ON '.$db->qn('u.id').' = '.$db->qn('a.created_by'))
			->join('LEFT', $db->qn('#__k2_rating','v').' ON '.$db->qn('a.id').' = '.$db->qn('v.itemID'))
			->where($db->qn('a.id').' = '.$db->q($id));
		$db->setQuery($query);
		if ($article = $db->loadObject()) {
			return $article;
		}
		
		return false;
	}
	
	protected function getValue($data, $name, $default = null) {
		if (isset($data[$name]) && !empty($data[$name])) {
			return $data[$name];
		}
		
		return $default;
	}
	
	protected function json_options() {
		if (version_compare(phpversion(), '5.4.0', '<')) {
			return 0;
		}
		
		return JSON_PRETTY_PRINT;
	}
}