<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Compiles the less files with new variables
 */
class RSTroneCompiler
{
	protected $id;
	protected $data;

	/**
	 * @param $id
	 * @param $data
	 */
	public function compile($id, $data)
	{
		try
		{

			if (isset($data['params']))
			{
				$this->compileLess($id, $data['params']);
			}

		} catch (Exception $e)
		{
			$app = JFactory::getApplication();

			if ($app->isAdmin())
			{
				$app->enqueueMessage(JText::_('TPL_RSTEMPLATE_LESS_COMPILE_ERROR') . '<br/>' . $e->getMessage(), 'warning');
			}
			else
			{
				echo json_encode(array('status' => false, 'message' => $e->getMessage()));
			}

		}

	}

	/**
	 * @param       $id
	 * @param array $newValues
	 *
	 * @throws Exception
	 */
	protected function compileLess($id, $newValues = array())
	{
		// Sanitize values
        $originalNewValues['googleContentFont'] = $newValues['googleContentFont'];
        $originalNewValues['googleTitleFont']   = $newValues['googleTitleFont'];
		$originalNewValues['googleNavbarFont']  = $newValues['googleNavbarFont'];
		
		$newValues = $this->compileLessFiles($id, $newValues);
		
		// Save the actual value of the font in the database
		$newValues = array_merge($newValues, $originalNewValues);

		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_templates/tables');
		$table = JTable::getInstance('Style', 'TemplatesTable', array());
		$table->load($id);

		$registry = new JRegistry;
		$params   = $registry->loadString($table->params)->toArray();
		$params   = array_merge($params, $newValues);

		$data = array(
			'home'   => $table->home,
			'params' => $params,
		);

		$table->bind($data);
		$table->store();

		echo json_encode(array('status' => true));
	}
	
	public function compileLessFiles($id, $newValues)
	{
		jimport('joomla.filesystem.file');

		// Load the Lessphp library
		require_once JPATH_PLUGINS . '/system/rstrone/classes/less/Autoloader.php';
		Less_Autoloader::register();
		
		// Get template style
		$parser 	= new Less_Parser(array('compress' => true));
		$newValues	= $this->getSanitizedValues($newValues);

		$parser->parseFile(JPATH_SITE . '/templates/rstrone/css/master/template.less', JUri::root() . 'templates/rstrone');
		$parser->ModifyVars($newValues);

		$css = $parser->getCss();
		
		// Get RSPageBuilder! elements style
		$elements_parser = new Less_Parser(array('compress' => true));
		
		$elements_parser->parseFile(JPATH_SITE . '/templates/rstrone/css/master/rsextensions/components/com_rspagebuilder/com_rspagebuilder_admin_style.less', JUri::root() . 'templates/rstrone');
		$elements_parser->ModifyVars($newValues);
		
		$elements_css = $elements_parser->getCss();

		// Save template style
		if (!JFile::write(JPATH_SITE . '/templates/rstrone/css/themes/template-' . $id . '.css', $css))
		{
			throw new Exception(JText::sprintf('RSTEMPLATE_COULD_NOT_WRITE_TO_FILE', $css));
		}
		
		// Save RSPageBuilder! elements style
		if (!JFile::write(JPATH_ROOT . '/media/com_rspagebuilder/css/admin-elements-style.css', $elements_css))
		{
			throw new Exception(JText::sprintf('RSTEMPLATE_COULD_NOT_WRITE_TO_FILE', $elements_css));
		}

		return $newValues;
	}
	
	protected function getSanitizedValues($newValues)
	{
		$values = array(
            // Colors
            'primary_color'         => '',
            'secondary_color'       => '',
            'body_bg_color'         => '',
            'text_color'            => '',
            'heading_color'         => '',
            'link_color'            => '',
            'navbar_bg_color'       => '',
            'navbar_text_color'     => '',
            'footer_bg_color'       => '',
            'footer_text_color'     => '',

            // Visual
            'border_radius'         => '',
            'button_border_radius'  => '',

            // Font
            'font_size_base'        => '',
            'font_size_navbar'      => '',
            'googleContentFont'   => '',
            'googleTitleFont'     => '',
            'googleNavbarFont'    => ''
		);

		$newValues = array_intersect_key($newValues, $values);
		
		// Sanitize fonts
        $newValues['googleContentFont'] = current(explode(':', $newValues['googleContentFont']));
        $newValues['googleTitleFont']   = current(explode(':', $newValues['googleTitleFont']));
		$newValues['googleNavbarFont']  = current(explode(':', $newValues['googleNavbarFont']));
        $newValues['googleContentFont'] = str_replace('+', ' ', $newValues['googleContentFont']);
        $newValues['googleTitleFont']   = str_replace('+', ' ', $newValues['googleTitleFont']);
		$newValues['googleNavbarFont']  = str_replace('+', ' ', $newValues['googleNavbarFont']);
		
		return $newValues;
	}
}
