/*
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

;
(function ($, window, document, undefined) {

	// Create the defaults once
	var pluginName = "megamenu",
		defaults = {
			template_id: null
		};

	// The actual plugin constructor
	function Plugin(element, options) {
		this.element = element;
		this.options = $.extend({}, defaults, options);

		this._defaults = defaults;
		this._name = pluginName;

		this.init();
	}

	Plugin.prototype = {
		init: function () {
			// Object for the settings we will send
			options = this.options;

			var $context = this;

			this.addLayout($context);
			this.fireModal($context);
			this.saveModal($context);
			this.clearPosition($context);

		},

		/**
		 * Function to save the layout of the mega menu as a JSON
		 * triggering a function to render a preview
		 *
		 * @param $context
		 */
		addLayout: function ($context) {
			var layouts = {
				'1col'  : ['span12'],
				'2col'  : ['span6', 'span6'],
				'2col-1': ['span4', 'span8'],
				'2col-2': ['span8', 'span4'],
				'3col'  : ['span4', 'span4', 'span4'],
				'4col'  : ['span3', 'span3', 'span3', 'span3']
			};

			$('.layout-type').click(function () {
				var $layout_type = $(this).attr('data-layout');
				var $classes = layouts[$layout_type];

				$context.renderLayout($classes, $context);

				var args = {
					'action': 'size'
				};

				$context.saveJson($classes, args);
			});
		},

		/**
		 * Function to render a preview of the mega menu's layout
		 *
		 * @param type
		 * @param context
		 */
		renderLayout: function (type, context) {
			var $cols = type;
			var $container = context.element;
			$($container).find('.row-fluid').empty();

			var $input = $('#jform_params_mega_menu_builder');

			var json = $input.val();
			json = $.parseJSON(json);

			$.each($cols, function (index) {
				$($container).find('.row-fluid').append(
					'<div class="' + this + ' col" data-col="' + (index + 1) + '">' +
					'<a href="#addModule" role="button" data-toggle="modal"  class="column-setting" data-col="' + (index + 1) + '"><span class="icon-plus"></span></a>' +
					'<span id="rstpl-internal-' + (index + 1) + '" class="module-position">' + json.cols[(index + 1)].content + '</span></div>'
				);
			});
		},

		/**
		 * Hook into the displaying of the modal to populate
		 * the SELECT where it's needed.
		 *
		 */
		fireModal: function (context) {
			$(context.element).on('click', '.column-setting', function () {
				var col = $(this).attr('data-col');
				$('#addModuleLabel').html("Editing position " + col);
				$('#keyholder').val(col);

				var $input = $('#jform_params_mega_menu_builder'),
					json = $input.val(),
					select = $('#moduleholder');

				json = $.parseJSON(json);
				select.val(json.cols[col].content);
				select.trigger("liszt:updated");
			});
		},

		/**
		 * Function to save the content of the column in the json
		 * the select is being populated from a list of active modules
		 * from the website
		 *
		 * @param context
		 */
		saveModal: function (context) {
			$('.saveObject').click(function () {
				var col = $('#keyholder'),
					select = $('#moduleholder').val(),
					args = {
						'action': 'col',
						'col'   : col.val(),
						'value' : select
					},
					data = null;

				context.saveJson(data, args);
				col.val('');

				jQuery('#rstpl-internal-' + args['col']).html(args.value+' <a class="text-error clearObject" href="javascript:void(0);" data-col="'+ args['col'] +'"><small class="icon-remove"></small></a>');

				jQuery('#addModule').modal('hide');
			});
		},
		/**
		 * Function to clear the selected position of the column
		 *
		 * @param context
		 */
		clearPosition: function (context) {
			jQuery('.module-position').on('click', '.clearObject', function () {

				var col = $('#keyholder');

					args = {
						'action': 'col',
						'col'   : jQuery(this).data('col'),
						'value' : ''
					},
					data = null;

				context.saveJson(data, args);
				col.val('');
				
				jQuery('#rstpl-internal-' + args['col']).html(args.value);
			});
		},
		
		/**
		 * Helper function to save the data in the JSON
		 * accepts 2 cases - size / content
		 * the content is being generated from Joomla! Modules
		 *
		 *
		 * @todo More features will be added in the future
		 *
		 * @param data
		 * @param args
		 */
		saveJson: function (data, args) {
			var $input = $('#jform_params_mega_menu_builder');

			var json = $input.val();
			json = $.parseJSON(json);

			switch (args.action) {

				case 'size':
					var $classes = data;

					$.each($classes, function(index) {
						json.cols[index+1].size = $classes[index];
					});

					for (i = $classes.length + 1; i <= 4; i++) {
						json.cols[i].size = 'rem';
					}

					break;

				case 'col':
					json.cols[args.col].content = args.value;
					break;

			}

			$input.val(JSON.stringify(json));
		}

	};


// Plugin wrapper around the constructor,
// preventing against multiple instantiations
	$.fn[pluginName] = function (options) {
		return this.each(function () {
			if (!$.data(this, "plugin_" + pluginName)) {
				$.data(this, "plugin_" + pluginName,
					new Plugin(this, options));
			}
		});
	};

})(jQuery, window, document);