<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('JPATH_PLATFORM') or die;

/**
 * Class JFormFieldMegaMenu
 */
class JFormFieldMegaMenu extends JFormField
{
	/**
	 * @var string
	 */
	protected $type = 'MegaMenu';

	/**
	 * @param $string
	 *
	 * @return string
	 */
	public function escape($string)
	{
		$escaped = htmlentities($string, ENT_COMPAT, 'utf-8');

		return $escaped;
	}

	/**
	 * Function that returns an array of modules that are published on your website
	 *
	 * @return array
	 */
	protected function getPositionsInUse()
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select($db->qn(array('a.title', 'a.position', 'a.id', 'b.menuid')))
			->from($db->qn('#__modules', 'a'))
			->join('LEFT', $db->quoteName('#__modules_menu', 'b') . ' ON (' . $db->quoteName('a.id') . ' = ' . $db->quoteName('b.moduleid') . ')')
			->where($db->qn('client_id') . ' = ' . $db->q('0') . ' AND ' . $db->qn('published') . ' = ' . $db->q(1));
		$db->setQuery($query);

		$result  = $db->loadAssocList();
		$modules = array();
		foreach ($result as $module)
		{
			$module['modules'] = array(
				$module['id'] => $module['title']
			);

			unset($module['id']);
			unset($module['title']);

			if (array_key_exists($module['position'], $modules))
			{
				$modules[$module['position']]['modules'] = array_replace_recursive($modules[$module['position']]['modules'], $module['modules']);
				array_filter($modules[$module['position']]['modules']);
			}
			else
			{
				$modules[$module['position']] = $module;
			}
		}

		unset($result);

		return $modules;
	}

	/**
	 * @return string
	 */
	public function getInput()
	{
		$document = JFactory::getDocument();
		$root     = JUri::root(true);
		$json     = json_decode($this->value);
		$modules  = $this->getPositionsInUse();

		JHtml::_('jquery.framework');
		$document->addScript($root . '/templates/rstrone/classes/fields/js/jquery-ui.js');
		$document->addScript($root . '/plugins/system/rstrone/fields/js/megamenu.js');
		$document->addStyleSheet($root . '/plugins/system/rstrone/fields/css/megamenu.css');
		$document->addScriptDeclaration('
			jQuery(window).load(function(){
				jQuery(".rs-megamenu-builder").megamenu();
			});
		');

		$html = '';
		$html .= '<div class="rs-megamenu-builder container">';
		$html .= '<div class="dropdown"><a class="dropdown-toggle" data-toggle="dropdown" href="#"><span class="icon-grid-view-2"></span></a>
					<ul class="dropdown-menu">
						<li><a href="javascript:void(0)" class="layout-type" data-layout="1col">'. JText::_('PLG_SYSTEM_RSTRONE_MEGA_MENU_ONE_COLUMN') .'</a></li>
						<li><a href="javascript:void(0)" class="layout-type" data-layout="2col">'. JText::_('PLG_SYSTEM_RSTRONE_MEGA_MENU_TWO_COLUMN') .'</a></li>
						<li><a href="javascript:void(0)" class="layout-type" data-layout="2col-1">'. JText::_('PLG_SYSTEM_RSTRONE_MEGA_MENU_TWO_COLUMN_VARIATION_A') .'</a></li>
						<li><a href="javascript:void(0)" class="layout-type" data-layout="2col-2">'. JText::_('PLG_SYSTEM_RSTRONE_MEGA_MENU_TWO_COLUMN_VARIATION_B') .'</a></li>
						<li><a href="javascript:void(0)" class="layout-type" data-layout="3col">'. JText::_('PLG_SYSTEM_RSTRONE_MEGA_MENU_THREE_COLUMN') .'</a></li>
						<li><a href="javascript:void(0)" class="layout-type" data-layout="4col">'. JText::_('PLG_SYSTEM_RSTRONE_MEGA_MENU_FOUR_COLUMN') .'</a></li>
					</ul>
				</div>';
		$html .= '<div class="row-fluid">';

		foreach ($json->cols as $i => $column)
		{
			if ($column->size != 'rem')
			{
				$html .= '
					<div class="' . $column->size . ' col" data-col="'.$i.'">
						<a href="#addModule" role="button" class="column-setting" data-toggle="modal" data-col="'.$i.'">
							<span class="icon-plus"></span>
						</a>
						<span id="rstpl-internal-' . $i . '" class="module-position">';
						if (!empty($column->content))
							$html .= $column->content.' <a class="text-error clearObject" href="javascript:void(0);" data-col="'.$i.'"><small class="icon-remove"></small></a>';
				$html .= '</span>
					</div>';
			}
		}

		$html .= '</div>';
		$html .= '<input id="' . $this->id . '" name="' . $this->name . '" type="hidden" value="' . $this->escape($this->value) . '">';
		$html .= '</div>';

		$html .= '<div id="addModule" class="modal hide fade" tabindex="-1">
					<div class="modal-header"><button class="close" type="button" data-dismiss="modal">×</button>
						<h3 id="addModuleLabel"> </h3>
					</div>';
		$html .= '<div class="modal-body"><select id="moduleholder">';

		foreach ($modules as $module) {
			$html .= '<option value="'.$module['position'].'">'.$module['position'].'</option>';
		}

		$html .= '</select></div>
					<input id="keyholder" type="hidden" />
					<div class="modal-footer">
						<a class="btn btn-primary saveObject" >' . JText::_("JSAVE") . '</a>
					</div>
				</div>';

		return $html;
	}
}
