<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

require_once JPATH_PLUGINS . '/system/rstrone/helper.php';

if (file_exists(JPATH_SITE . '/templates/rstrone/templateDetails.xml')) {
	$xml = simplexml_load_file(JPATH_SITE . '/templates/rstrone/templateDetails.xml');
} else {
	return false;
}

if ($xml->author != 'RSJoomla!')
{
	return false;
}

/**
 * Add In-Depth Customization For The Template
 */
class plgSystemRSTrone extends JPlugin
{
	public function __construct(& $subject, $config)
	{
		if ($position = JFactory::getApplication()->input->get('rstplpos',''))
			JFactory::getSession()->set('rstplpos',$position);

		parent::__construct($subject, $config);
	}

	/**
	 * Used to recompile the less files
	 * @method onExtensionAfterSave
	 *
	 * @param  string  $context
	 * @param  string  $table
	 * @param  boolean $isNew
	 *
	 */
	public function onExtensionAfterSave($context, $table, $isNew)
	{
		if ($context == 'com_templates.style')
		{
			$app	= JFactory::getApplication();
			$id     = $app->input->get('id');

			if (RSTroneHelper::getTemplateType($id) == 'rstrone')
			{
				require_once JPATH_PLUGINS . '/system/rstrone/classes/rscompiler.php';
				
				$data		= $app->input->get('jform', array(), 'array');
				$compiler	= new RSTroneCompiler();
				$compiler->compile($id, $data);
			}
		}
	}
	
	/**
	 * Event that renders the customizer
	 * @method onRSTemplateCustomizerDisplay
	 *
	 * @return string
	 */
	public function onRSTemplateCustomizerDisplay()
	{
		$app = JFactory::getApplication();

		if ($app->isAdmin())
		{
			return false;
		}

		if (RSTroneHelper::getTemplateType($app->getTemplate(true)->id) == 'rstrone')
		{
			$element = new JLayoutFile('layouts.customizer', JPATH_PLUGINS . '/system/rstrone', array('debug' => false));
			echo $element->render((object) array());
		}
	}

	/**
	 * Render the customizer
	 */
	public function onAfterRoute()
	{
		$app  = JFactory::getApplication();
		$user = JFactory::getUser();
		$doc  = JFactory::getDocument();
		if ($app->isAdmin() && $app->input->get('option') == 'com_templates')
		{
			$id = $app->input->getInt('id');
		}
		elseif ($app->isSite())
		{
			$id = $app->getTemplate(true)->id;
		}
		
		if (!empty($id) && (RSTroneHelper::getTemplateType($id) == 'rstrone'))
		{
			$doc->addScriptDeclaration('
			var rs_root			= "' . JUri::root(true) . '";
			var rs_base_path	= "' . JUri::root(true) . '/templates/rstrone";
			');
			
			// Grab the task
			$task    = $app->input->get('rstpl_task', '');
			$session = JFactory::getSession();

			if ($task == 'close')
			{
				$session->clear('customizer');
			}

			if ($task == 'customizer' && !$app->isAdmin())
			{
				$session->set('customizer', 'open');
			}

			if ($task == 'compile' && $user->authorise('core.manage', 'com_templates'))
			{
				require_once JPATH_PLUGINS . '/system/rstrone/classes/rscompiler.php';
				
				// Store it
				$jinput   = $app->input;
				$data     = array('params' => $jinput->getArray());
				$compiler = new RSTroneCompiler();
				$compiler->compile($id, $data);
				$app->close();
			}
		}
	}

	/**
	 * Add the additional menu parameters (for mega-menu / icon)
	 * @method onContentPrepareForm
	 *
	 * @param  object $form
	 * @param  object $data
	 *
	 * @return boolean
	 */
	public function onContentPrepareForm($form, $data)
	{
		if (RSTroneHelper::getDefaultSiteTemplate() == 'rstrone')
		{
			if (!($form instanceof JForm))
			{
				$this->_subject->setError('JERROR_NOT_A_FORM');

				return false;
			};
			$name  = $form->getName();
			$forms = array(
				'com_menus.item' => 'menu'
			);

			if (!isset($forms[$name]))
			{
				return true;
			};

			if ($forms[$name] == 'menu')
			{
				$this->loadLanguage();
				JForm::addFormPath(JPATH_PLUGINS . '/system/rstrone/forms');
				$form->loadFile($forms[$name], false);
			}
		}
		return true;
	}
	
	/**
	*	Add modules from Template Positions
	*   Compile less files on frontend refresh when DevMode enabled
	**/
	public function onAfterRender()
	{
		$app			= JFactory::getApplication();
		$input			= $app->input;
		$template_type	= $app->isAdmin() ? RSTroneHelper::getDefaultSiteTemplate() : RSTroneHelper::getTemplateType($app->getTemplate(true)->id);

		if ( $template_type == 'rstrone')
		{
			// Frontend - compile less files when devmode is On
			if (!$app->isAdmin())
			{
				$template_id = $app->getTemplate(true)->id;
				$data 		 = RSTroneHelper::getTemplateParams($template_id);
				if (!empty($data['dev_mode'])) {
					require_once JPATH_PLUGINS . '/system/rstrone/classes/rscompiler.php';

					$compiler = new RSTroneCompiler();
					$compiler->compileLessFiles($template_id, $data);
				}
			}

			// Add position to module type links list in order to populate the position field
			if (
				$app->isAdmin() && 
				$input->get('rstplpos','') && 
				$input->get('option','') == 'com_modules' &&
				$input->get('view','') == 'select'
			) {
				$contents = JResponse::getBody();

				$contents = str_replace('com_modules&amp;task=module.add', 'com_modules&amp;task=module.add&amp;rstplpos='.$input->get('rstplpos',''), $contents);
				JResponse::setBody($contents);
			}

			// Populate form position when adding module from Template Positions
			if (
				$input->get('option', '') == 'com_modules' && 
				$input->get('view', '') == 'module' && 
				$input->get('layout', '') == 'edit' &&
				$input->getInt('id', '') == 0
			) {
				if ($position = JFactory::getSession()->get('rstplpos','')) {
					$contents = JResponse::getBody();
					$script = '
						<script type="text/javascript">
						jQuery(document).ready(function(){
							jQuery("#jform_position").val("'.$position.'");
							jQuery("#jform_position").trigger("liszt:updated");
						});
						</script>';

					$contents = str_replace( '</head>', $script.'</head>', $contents );
					JResponse::setBody($contents);
				}
				JFactory::getSession()->clear('rstplpos');
			}
		}
	}
}
