<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('JPATH_PLATFORM') or die;

if (! defined('RSTEMPLATE_PATH')) {
	define('RSTEMPLATE_PATH', dirname(dirname(dirname(__FILE__))));
}

require_once dirname(dirname(__FILE__)) . '/params.php';

/**
 * Extends Joomla! form field, enabling us to configure the template positions
 */
class JFormFieldDefinedPositions extends JFormField
{
	/**
	 * @var string
	 */
	protected $type = 'position_changer';

	/**
	 * Escape function
	 * @method escape
	 *
	 * @param  string $string
	 *
	 * @return string
	 */
	public function escape($string)
	{
		$escaped = htmlentities($string, ENT_COMPAT, 'utf-8');

		return $escaped;
	}

	/**
	 * Function that returns an array of modules that are published on your website
	 *
	 * @return array
	 */
	protected function getPositionsInUse()
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select($db->qn(array('a.title', 'a.position', 'a.id', 'b.menuid')))
			->from($db->qn('#__modules', 'a'))
			->join('LEFT', $db->quoteName('#__modules_menu', 'b') . ' ON (' . $db->quoteName('a.id') . ' = ' . $db->quoteName('b.moduleid') . ')')
			->where($db->qn('client_id') . ' = ' . $db->q('0') . ' AND ' . $db->qn('published') . ' = ' . $db->q(1));
		$db->setQuery($query);

		$result  = $db->loadAssocList();
		$modules = array();
		foreach ($result as $module)
		{
			$module['modules'] = array(
				$module['id'] => $module['title']
			);

			unset($module['id']);
			unset($module['title']);

			if (array_key_exists($module['position'], $modules))
			{
				$modules[$module['position']]['modules'] = array_replace_recursive($modules[$module['position']]['modules'], $module['modules']);
				array_filter($modules[$module['position']]['modules']);
			}
			else
			{
				$modules[$module['position']] = $module;
			}
		}

		unset($result);

		return $modules;
	}

	/**
	 * Used to get the row settings
	 * @method getRowSettings
	 *
	 * @return object
	 */
	protected function getRowSettings()
	{
		// Get the style ID and get a connection to the database
		$styleId = JFactory::getApplication()->input->getInt('id');
		$db      = JFactory::getDbo();
		$query   = $db->getQuery(true);
		// Start Building the query
		$query->select($db->qn('params'));
		$query->from($db->qn('#__template_styles'));
		$query->where($db->qn('id') . ' = ' . $db->q($styleId));
		$db->setQuery($query);
		// Decode the results as a JSON
		$params = json_decode($db->loadResult($query));

		// Decode the row setting params and return the object
		return $params->defined_row_settings;
	}

	/**
	 * Rendering the field
	 * @method getInput
	 *
	 * @return string
	 */
	protected function getInput()
	{
		// Get a reference to the document
		$document = JFactory::getDocument();
		$root     = JUri::root(true);
		$addClass = null;
		// Add the necessary JS and CSS
		$document->addScript($root . '/templates/rstrone/classes/fields/js/jquery-ui.js');
		$document->addScript($root . '/templates/rstrone/classes/fields/js/rs-positions.js');
		$document->addStyleSheet($root . '/templates/rstrone/classes/fields/css/rs-positions.css');
		$html = null;

		$activeModules = $this->getPositionsInUse();
		// Get the row settings
		try
		{

			if ($this->value != '')
			{
				$positions = explode(',', $this->value);
			}

			$rowSettings = @json_decode($this->getRowSettings());
			// If this is not a JSON we throw an error

			if ($rowSettings === null)
			{
				// we provide a button for the user to revert to the default row settings
				echo '<a class="btn" id="rwJSON">' . JText::_('LIB_RSTEMPLATE_DEFAULT_ROW_SETTINGS') . '</a>';
				throw new Exception(JText::_('LIB_RSTEMPLATE_COULD_NOT_DECODE_JSON_DATA'));
			}
			else
			{
				$params  = new RSParams($this->form->getFieldset());
				$element = new JLayoutFile('layouts.pagebuilder', RSTEMPLATE_PATH . '/classes/fields', array('debug' => false));

				$definedPosition = array(
					'positions'     => $positions,
					'rowsettings'   => $rowSettings,
					'removed'       => $params->get('removed_positions'),
					'activemodules' => $activeModules
				);

				$html .= $element->render( (object) $definedPosition );

			$html .= '<input id="jform_params_defined_positions" type="hidden" value="' . $this->value . '" name="jform[params][defined_positions]">';
			}

		} catch (Exception $e)
		{
			$application = JFactory::getApplication();
			$application->enqueueMessage($e->getMessage(), 'error');
		}

		return $html;
	}

}
