<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

$pos           = $displayData->position; // Actual Position
$activeModules = $displayData->activemodules; // Active Modules on your website

$class = 'row sortable-cols';

/**
 * If there are 2 or more positions on a row, allow them to resize
 */
if (count((array) $displayData->rowsettings->$pos->cols) >= 2)
{
	$class .= ' resizable';
}
?>
<!-- Position Row -->
<div class="<?php echo $class ?>">
	<?php
	// Generate the columns of the row
	foreach ($displayData->rowsettings->$pos->cols as $cols => $properties):

		/**
		 * Check which positions have modules assigned and which are empty
		 */
		if (array_key_exists($cols, $activeModules))
		{

			$properties->modules = array();
			foreach ($activeModules[$cols]['modules'] as $id => $module)
			{
				$properties->modules[$id] = $module;
			}

		}
		else
		{
			$properties->module = 'empty';
		}

		/**
		 * Create an array that contains the HTML attributes of the column
		 */
		$positionAttr = array(
			'class'         => 'col ' . $properties->size,
			'data_computed' => $properties->size,
			'data_col'      => $cols,
			'style'         => '',
		);

		/**
		 * In case the column is removed, we need to hide it
		 */
		if ($properties->size == 'rem')
		{
			$positionAttr['style'] = 'display:none';
		}

		/**
		 * Build the HTML string (e.g. class="col span6" data_computed="span6" data_col="top-a")
		 */

		$attributes = '';
		foreach (array_filter($positionAttr) as $attr => $val)
		{
			$attributes .= $attr . '= "' . $val . '" ';
		}

		?>
		<!-- Position Column -->
		<div <?php echo $attributes ?>>
			<?php
			/**
			 * If position has widget attached, add an icon so we know that is not empty
			 */
			if ($properties->wg != 0)
			{
				?>
				<span class="icon-cube has-widget hasTooltip" data-original-title="<?php echo JText::_("TPL_RSTEMPLATE_ATTACHED_WIDGET") ?>"></span>
				<?php
			}
			/**
			 * If the position has module attached, we create a dropdown menu
			 * with links to the modules in question ( linking to the actual MODULE)
			 */
			if (!empty($properties->modules))
			{
				?>
				<!-- Modules Dropdown Menu -->
				<div class="dropdown">
					<a class="dropdown-toggle" data-toggle="dropdown" href="#"><span class="icon-enter has-module"></span></a>
					<?php 
					$list_items = '';
					$ModuleModals = '';
					foreach ($properties->modules as $id => $title) {
						$list_items .= '<li><a href="#EditModule'. $cols.$id.'" role="button" data-toggle="modal"><i class="icon-edit"></i> '.$title .'</a></li>';
						$ModuleModals .= '<script type="text/javascript">
											jQuery(document).ready(function(){
												jQuery("#saveModule'.$cols.$id.'").click(function() {
													var iframe = jQuery("#EditModule'.$cols.$id.' iframe").contents();

													iframe.find("#module-form input[name=\"task\"]").val("module.apply");
													iframe.find(".button-apply").trigger("click");
												});
											});
										   </script>'. 
										   JHtml::_(
											'bootstrap.renderModal',
											'EditModule' . $cols.$id,
											array(
													'title'		  => JText::sprintf("TPL_RSTEMPLATE_EDIT_MODULE", $title),
													'backdrop'    => 'static',
													'keyboard'    => false,
													'closeButton' => true,
													'url'         => JUri::root(true).'/administrator/index.php?option=com_modules&task=module.edit&id='.$id,
													'height'      => '400px',
													'width'       => '800px',
													'bodyHeight'  => '70',
													'modalWidth'  => '80',
													'footer'      => '<button id="saveModule'.$cols.$id.'" type="button" class="btn btn-primary" aria-hidden="true">'. JText::_("JAPPLY") . '</button>'
												)
											);
					}
					?>
					<ul class="dropdown-menu">
						<?php echo $list_items; ?>
					</ul>
					<?php echo $ModuleModals; ?> 
				</div>
				<?php
			}
			/**
			 * The content column requires a different approach
			 */
			if ($cols != 'content')
			{
				$link = JUri::root(true) .'/administrator/index.php?option=com_modules&view=select&rstplpos='.$cols;
			?>
				<!-- Add Module to position Button -->
				<script type="text/javascript">
				jQuery(document).ready(function(){
					jQuery("#saveModule<?php echo $cols;?>").click(function() {
						var iframe = jQuery("#ModulesPosition<?php echo $cols;?> iframe").contents();

						iframe.find("#module-form input[name=\"task\"]").val("module.apply");
						// Populate position in case the form has errors on submit
						jQuery("<input>").attr({
							type	: "hidden",
							id		: "rstplpos",
							name	: "rstplpos",
							value	: "<?php echo $cols;?>"
						}).appendTo(iframe.find("#module-form"));
						
						iframe.find(".button-apply").trigger("click");
					});
				});
				</script>
				<a class="add-new-module hasTooltip" data-original-title="<?php echo JText::sprintf("TPL_RSTEMPLATE_ADD_NEW_MODULE", $cols) ?>" href="#ModulesPosition<?php echo $cols;?>" role="button" data-toggle="modal"><span class="icon-plus"></span></a>
				<?php
				echo JHtml::_(
					'bootstrap.renderModal',
					'ModulesPosition' . $cols,
					array(
						'title'		  => JText::sprintf("TPL_RSTEMPLATE_ADD_NEW_MODULE", $cols),
						'backdrop'    => 'static',
						'keyboard'    => false,
						'closeButton' => true,
						'url'         => $link,
						'height'      => '400px',
						'width'       => '800px',
						'bodyHeight'  => '70',
						'modalWidth'  => '80',
						'footer'      => '<button id="saveModule'.$cols.'" type="button" class="btn btn-primary" aria-hidden="true">'. JText::_("JAPPLY") . '</button>',
					)
				);
			}
			
			if (count((array) $displayData->rowsettings->$pos->cols) >= 2) {
			?>
			<a class="col-handle"><i class="icon-move"></i></a>
			<?php }
			echo $cols;
			?>
			<!-- Column Settings Button -->
			<a class="customization-col hasTooltip" data-original-title="<?php echo JText::_("TPL_RSTEMPLATE_COLUMNSETTINGS") ?>" data_key="<?php echo $pos ?>" href="#colSettings" role="button" data-toggle="modal"><i class="icon-options"></i></a>
		</div>
		<!-- / Position Column -->
	<?php endforeach; ?>
</div>
<!-- / Position Row -->