<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('JPATH_PLATFORM') or die;
JFormHelper::loadFieldClass('list');

/**
 * Extends Joomla! Form Fields, adding possibility to select a google font to render content
 */
class JFormFieldRSFonts extends JFormFieldList
{
	/**
	 * @var string
	 */
	protected $type = 'RSFonts';

	/**
	 * Function to call the gfonts api
	 * @method getFonts
	 *
	 * @return array
	 */
	public static function getFonts()
	{
		try
		{
			$key = self::getGoogleKey();

			/**
			 * In case the user does not have / want to use an API KEY for google fonts
			 * we have a fallback stored locally.
			 */
			if (!$key)
			{
				$fonts = @file_get_contents(JPATH_ROOT . '/templates/rstrone/classes/fields/data/gfonts.json');

				if ($fonts === false)
				{
					throw new Exception(JText::_('LIB_RSTEMPLATE_COULD_NOT_LOAD_JSON'));
				}

				$fonts = json_decode($fonts);

				if ($fonts === null)
				{
					throw new Exception(JText::_('LIB_RSTEMPLATE_COULD_NOT_DECODE_JSON_DATA'));
				}

				$list = array();
				foreach ($fonts->items as $item)
				{

					if (!empty($item->family))
					{
						$list[] = array(
							'family'  => $item->family,
							'variant' => $item->variants
						);
					}

				}

				$application = JFactory::getApplication();
				$application->enqueueMessage(JText::_('TPL_RSTEMPLATE_GOOGLE_FONT_NOTICE'));

				return $list;
			}

			$url  = 'https://www.googleapis.com/webfonts/v1/webfonts?key=' . $key;
			$http = JHttpFactory::getHttp();

			if ($response = $http->get($url, null, 5))
			{

				if ($response->code != 200)
				{
					throw new Exception(JText::sprintf('LIB_RSTEMPLATE_CONNECTION_ERROR', $response->code));
				}

				$data = json_decode($response->body);

				if ($data === null)
				{
					throw new Exception(JText::_('LIB_RSTEMPLATE_COULD_NOT_DECODE_JSON_DATA'));
				}

				if (!isset($data->items))
				{
					throw new Exception(JText::_('LIB_RSTEMPLATE_JSON_DATA_CONTAINS_NO_ITEMS'));
				}

				$list = array();
				foreach ($data->items as $item)
				{

					if (!empty($item->family))
					{
						$list[] = array(
							'family'  => $item->family,
							'variant' => $item->variants
						);
					}

				}

				return $list;
			}

		} catch (Exception $e)
		{
			$application = JFactory::getApplication();
			$application->enqueueMessage($e->getMessage(), 'error');
		}

		return array();
	}

	/**
	 * we don't need to call the api every time, we store the results in cache
	 * @method getOptions
	 *
	 * @return object
	 */
	public function getOptions()
	{
		$cache = JFactory::getCache('lib_rstemplate');
		$cache->setCaching(true);
		$fonts = $cache->call(array('JFormFieldRSFonts', 'getFonts'));
		foreach ($fonts as $font)
		{
			$url       = str_replace(' ', '+', $font['family']) . ':' . implode(',', $font['variant']);
			$options[] = JHtml::_('select.option', $url, $font['family']);
		}

		return $options;
	}

	public static function getGoogleKey()
	{
		// Get the style ID and get a connection to the database
		$app     = JFactory::getApplication();
		$styleId = $app->input->getInt('id');

		if (!$app->isAdmin())
		{
			$styleId = $app->getTemplate(true)->id;
		}

		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		// Start Building the query
		$query->select($db->qn('params'));
		$query->from($db->qn('#__template_styles'));
		$query->where($db->qn('id') . ' = ' . $db->q($styleId));
		$db->setQuery($query);

		// Decode the results as a JSON
		$params = json_decode($db->loadResult($query));

		return $params->google_fonts_key;
	}
}