<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('JPATH_PLATFORM') or die;

/**
 * Extends Joomla! form fields, provides a carousel with available pre-defined themes
 */
class JFormFieldRSThemes extends JFormField
{
	/**
	 * @var string
	 */
	protected $type = 'RSThemes';

	/**
	 * Rendering the field
	 * @method getInput
	 *
	 * @return string
	 */
	public function getInput()
	{
		// Get the document so we can add the script
		$document = JFactory::getDocument();
		$document->addScript(JUri::root(true) . '/templates/rstrone/classes/fields/js/slider.js');
		$document->addScript(JUri::root(true) . '/templates/rstrone/classes/fields/js/rstemplate.js');
		$document->addStylesheet(JUri::root(true) . '/templates/rstrone/classes/fields/css/rsthemes.css');
		// Try getting the JSON contents
		try
		{
			// used @ to suppress the error
			$themes = @file_get_contents(JPATH_ROOT . '/templates/rstrone/css/themes/themes.json');

			// If the file is not there we throw an exception here
			if ($themes === false)
			{
				throw new Exception(JText::_('LIB_RSTEMPLATE_COULD_NOT_LOAD_JSON'));
			}

			// If no errors were encountered, we try to decode the json
			$themes = json_decode($themes);
			if ($themes === null)
			{
				throw new Exception(JText::_('LIB_RSTEMPLATE_COULD_NOT_DECODE_JSON_DATA'));
			}

			// We render the field if everything is ok
			$htmlField = '<div class="alert alert-success" style="display:none" id="rstpl-style-alert"><i class="icon-checkmark"></i> ' . JText::_('LIB_RSTEMPLATE_STYLES_APPLIED') . '</div>';
			$htmlField .= '<div id="' . $this->id . '" class="owl-carousel ' . $this->class . '">';
			foreach ($themes as $theme => $settings)
			{
				$htmlField .= '<div class="item">
	            <span class="bg" style="background-color:' . $settings->{'primary_color'} . '"></span>
	            <span class="bg" style="background-color:' . $settings->{'secondary_color'} . '"></span>
	            <a class="hasTooltip btn btn-mini btn-success populateInputs" title="' . $theme . '" href="javascript:void(0)" data-action="predefinedTheme" data-customizer="' . $theme . '" >'.JText::_('LIB_RSTEMPLATE_LOAD_THEME').'</a>
	            </div>';
			}

			$htmlField .= '</div>';
			$htmlField .= '<a href="' . JUri::root(true) . '/index.php?rstpl_task=customizer" target="_blank" class="btn btn-large"><span class="icon-brush"></span>' . JText::_('TPL_RSTEMPLATE_CUSTOMIZE_IT') . '</a>';

			return $htmlField;


		} // Handle the errors
		catch (Exception $e)
		{
			$application = JFactory::getApplication();
			$application->enqueueMessage($e->getMessage(), 'error');
		}

		return false;
	}

}
