<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Template Helper
 */
class RSTemplateHelper
{
	/**
	 * Shows the gravatar in the articles
	 * @method showGravatar
	 *
	 * @param  int $user_id
	 *
	 * @return string
	 */
	public static function showGravatar($user_id)
	{
		$uri             = JUri::getInstance();
		$prefix_protocol = 'http';

		if ($uri->isSSL())
		{
			$prefix_protocol .= 's';
		}

		$user   = JFactory::getUser($user_id);
		$email  = $user->get('email');
		$hash   = md5($email);
		$params = JFactory::getApplication()->getTemplate(true)->params;

		if ($params->get('show_gravatar', 1))
		{
			return '<img src="' . $prefix_protocol . '://www.gravatar.com/avatar/' . $hash . '?s=150&d=mm" class="article-img" />';
		}
		else
		{
			return '';
		}

	}

	/**
	 * Converts timestamps in readable format
	 * @method human_timing
	 *
	 * @param  string $time
	 *
	 * @return string
	 */
	public static function human_timing($time)
	{
		static $translations = null;

		if ($translations === null)
		{
			$translations = array(
				'year'    => JText::_('LIB_RSTEMPLATE_YEAR'),
				'years'   => JText::_('LIB_RSTEMPLATE_YEARS'),
				'month'   => JText::_('LIB_RSTEMPLATE_MONTH'),
				'months'  => JText::_('LIB_RSTEMPLATE_MONTHS'),
				'week'    => JText::_('LIB_RSTEMPLATE_WEEK'),
				'weeks'   => JText::_('LIB_RSTEMPLATE_WEEKS'),
				'day'     => JText::_('LIB_RSTEMPLATE_DAY'),
				'days'    => JText::_('LIB_RSTEMPLATE_DAYS'),
				'hour'    => JText::_('LIB_RSTEMPLATE_HOUR'),
				'hours'   => JText::_('LIB_RSTEMPLATE_HOURS'),
				'minute'  => JText::_('LIB_RSTEMPLATE_MINUTE'),
				'minutes' => JText::_('LIB_RSTEMPLATE_MINUTES'),
				'second'  => JText::_('LIB_RSTEMPLATE_SECOND'),
				'seconds' => JText::_('LIB_RSTEMPLATE_SECONDS'),
			);
		}

		$time = time() - $time; // to get the time since that moment

		$tokens = array(
			31536000 => 'year',
			2592000  => 'month',
			604800   => 'week',
			86400    => 'day',
			3600     => 'hour',
			60       => 'minute',
			1        => 'second'
		);

		foreach ($tokens as $unit => $translation_key)
		{

			if ($time < $unit)
			{
				continue;
			}

			$number_of_units = floor($time / $unit);

			return $number_of_units . ' ' . $translations[$translation_key . (($number_of_units > 1) ? 's' : '')];
		}

		return false;

	}

	/**
	 * Useful in the row-settings to determine if the array has certain keys
	 * @method filter_by_key
	 *
	 * @param  array  $array
	 * @param  string $key
	 *
	 * @return string
	 */
	public static function filter_by_key($array, $key)
	{
		foreach ($array as $k)
		{

			if (isset($k->$key))
			{

				if ($k->$key)
				{
					return $k->$key;
				}

			}

		}

		return false;
	}

	/**
	 * Add attributes to HTML tags from an array
	 * @method add_attributes
	 *
	 * @param  array $array [description]
	 *
	 * @return mixed
	 */
	public static function add_attributes(array $array)
	{
		$attrbs = array();
		foreach ($array as $property => $value)
		{

			if (strlen($value))
			{
				$attrbs[] = $property . '="' . self::esc_html($value) . '"';
			}

		}

		if ($attrbs)
		{
			return implode(' ', $attrbs);
		}

		return false;
	}

	/**
	 * Function to escape HTML
	 * @method esc_html
	 *
	 * @param  string $string
	 *
	 * @return string
	 */
	public static function esc_html($string)
	{
		return htmlspecialchars($string, ENT_QUOTES, 'UTF-8');
	}
}


