<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

define('_JEXEC', 1);

define('JPATH_BASE', realpath(dirname(__FILE__) . '/../../../administrator'));

require_once JPATH_BASE . '/includes/defines.php';
require_once JPATH_BASE . '/includes/framework.php';
require_once JPATH_PLUGINS . '/system/rstrone/helper.php';
require_once JPATH_PLUGINS . '/system/rstrone/classes/rscompiler.php';

require_once dirname(__FILE__) . '/parser/xmlsorter.php';
require_once dirname(__FILE__) . '/parser/xmlparser.php';

$templateName = XMLParser::getTemplateName();

// Instantiate the application.
$app = JFactory::getApplication('administrator');

// Load language
JFactory::getLanguage()->load('tpl_' . $templateName, JPATH_SITE, 'en-GB', true);

try
{
	$user = JFactory::getUser();

	// Check if the user is logged in
	if ($user->get('guest'))
	{
		throw new Exception(JText::_('TPL_NOT_LOGGED_IN_ERROR'));
	}

	// Check if the user can manage com_templates
	if (!$user->authorise('core.manage', 'com_templates')
		|| !$user->authorise('core.edit', 'com_templates')
		|| !$user->authorise('core.create', 'com_templates')
		|| !$user->authorise('core.admin', 'com_templates')
	)
	{
		throw new Exception(JText::_('TPL_NOT_ADMIN_ERROR'));
	}

	$db    = JFactory::getDbo();
	$query = $db->getQuery(true);

	$query->select($db->qn(array('params')))
		->from($db->qn('#__template_styles'))
		->where($db->qn('template') . ' = ' . $db->q($templateName));
	$db->setQuery($query);
	$templateParams = $db->loadObjectList();

	$params = '';
	$params = $templateParams[0]->params;
	$params = json_decode($params);

	if ($params->installed_sample_data != '0')
	{
		throw new Exception(JText::_('TPL_ALREADY_INSTALLED'));
	};

	$check       = $app->input->get('check');
	$extensions  = $app->input->get('extensions', array(), 'array');
	$sample_type = $app->input->get('sample_type', '', 'string');

	if (!$sample_type)
	{
		throw new Exception(JText::_('TPL_NO_SAMPLE_TYPE'));
	}

	if ($check == 'extension')
	{
		if ($extensions)
		{
			$missing = array();
			foreach ($extensions as $extension)
			{
				if (!XMLParser::checkIfInstalled($extension))
				{
					$missing[] = '<i style="color: red">' . $extension . '</i>';
				}
			}

			if ($missing)
			{
				throw new Exception(JText::sprintf(JText::_('TPL_FOR_BEST') . '<br />' . implode(JText::_('TPL_AND') . ' ', $missing) . '<br /><br /><button type="button" class="btn btn-primary" onclick="installSampleData(\'force\', true)">' . JText::_('TPL_PROCEED_ANYWAY') . '</button>'));
			}
		}
		else
		{
			throw new Exception(JText::_('TPL_NO_EXTENSION'));
		}
	}

	// Remove default home
	$query->clear();
	$query->update($db->qn('#__template_styles'))
		->set($db->qn('home') . ' = ' . $db->q(0))
		->where($db->qn('client_id') . ' = ' . $db->q(0))
		->where($db->qn('title') . ' != ' . $db->q('RSTrone! - Default'));
	$db->setQuery($query);
	$db->execute();

	// Set RSTrone! Default before parsers, may be overwritten later on
	$query->clear();
	$query->update($db->qn('#__template_styles'))
		->set($db->qn('home') . ' = ' . $db->q(1))
		->where($db->qn('title') . ' = ' . $db->q('RSTrone! - Default'));
	$db->setQuery($query);
	$db->execute();

	$sorter = new XMLSorter($sample_type);
	$files	= $sorter->getFiles();
	
	foreach ($files as $file) {
		$parser = XMLParser::getInstance($file, $sample_type);
		
		if ((substr($file, 0, 2) == 'k2')) {
			$parser->saveItem($file, $sample_type);
		} else {
			$items = $parser->getItems();
			
			foreach ($items as $item) {
				$parser->saveItem($item, $sample_type);
			}
		}
	}
	
	// Add RSPageBuilder! modules ids
	if (in_array('rspagebuilder', $files)) {
		$parser = XMLParser::getInstance('rspagebuilder', $sample_type);
		$items  = $parser->getItems();
		
		foreach ($items as $item) {
			$parser->addModulesIds($item, $sample_type);
		}
	}
	
	// Compile RSTrone less files
	$query->clear();
	$query->select($db->qn('id'))
		->from($db->qn('#__template_styles'))
		->where($db->qn('template') . ' = ' . $db->q($templateName));
	$db->setQuery($query);
	$templates = $db->loadObjectList();
	
	foreach ($templates as $template) {
		$id			= $template->id;
		$data		= RSTroneHelper::getTemplateParams($id);
		$compiler	= new RSTroneCompiler();
		
		$compiler->compileLessFiles($id, $data);
	}
	
	die(json_encode(array(
		'success' => '<span style="color: green">' . JText::_('TPL_SUCCESS_DATA') . '</span>'
	)));
} catch (Exception $e)
{
	die(json_encode(array(
		'error' => $e->getMessage()
	)));
}
