<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Add Mega Menu Functionality
 */
class RSMegaMenu
{
	/**
	 * @var JRegistry
	 */
	protected $data;

	/**
	 * Creates a new JRegistry Object
	 * @method __construct
	 *
	 * @param  object $data
	 */
	public function __construct($data)
	{
		$params 		= $data->params;
		$this->params 	= new JRegistry($params);
	}

	/**
	 * Builds the container of the menu
	 * @method buildMegaMenu()
	 *
	 * @return string
	 */
	public function buildMegaMenu()
	{
		$lang      = JFactory::getLanguage();
		$extension = 'com_content';
		$lang->load($extension);

		$mega_menu = json_decode($this->params->get('mega_menu_builder'));

		if ($mega_menu == null)
		{
			return false;
		}

		$cols = array();
		foreach ($mega_menu->cols as $col)
		{
			if ($col->size != 'rem')
			{
				$cols[] = $col;
			}
		}

		if (!empty($cols))
		{
			$html         = '';
			$desktop_only = '';

			if ((bool) $this->params->get('mega_menu_visible'))
			{
				$desktop_only = ' visible-desktop';
			}

			$html .= '<div class="rstpl-mega-menu-container' . $desktop_only . '">';
			$html .= '<div class="row-fluid">';

			foreach ($cols as $col)
			{
				$html .= '<div class="' . $col->size . '">' . (empty($col->content) ? '' : $this->megaMenu('{loadposition ' . $col->content . '}')) . '</div>';
			}

			$html .= '</div></div>';

			return $html;
		}

		return false;

	}

	/**
	 * Checks the obj for the {loadposition} syntax and replaces it with actual content
	 * @method megaMenu
	 *
	 * @param $obj
	 *
	 * @return mixed
	 */
	public function megaMenu($obj)
	{
		$regex = '/{loadposition\s+(.*?)}/i';
		$title = null;
		preg_match_all($regex, $obj, $matches, PREG_SET_ORDER);

		if ($matches)
		{
			foreach ($matches as $match)
			{
				$matcheslist = explode(',', $match[1]);
				$position    = trim($matcheslist[0]);
				$output      = $this->_load($position);
				$obj         = preg_replace("|$match[0]|", addcslashes($output, '\\$'), $obj, 1);
			}

			return $obj;
		}

		return $obj;
	}

	/**
	 * Replaces the {loadposition} syntax with the module content
	 * @method _load
	 *
	 * @param $position
	 *
	 * @return string
	 */
	public function _load($position)
	{
		$document = JFactory::getDocument();
		$renderer = $document->loadRenderer('module');
		$modules  = JModuleHelper::getModules($position);

		ob_start();
		foreach ($modules as $module)
		{
			echo $renderer->render($module);
		}

		$content = ob_get_clean();

		return $content;
	}

}
