<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');
define('K2_JVERSION', '30');

JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_k2/tables');

/**
 * Class XMLParserK2
 */
class XMLParserK2 extends XMLParser {
	
	protected function getTable($table) {
		return JTable::getInstance($table, 'Table');
	}

	public function saveItem($file, $folder) {
		$check = XMLParser::checkIfInstalled('com_k2');
		
		if ($check) {
			switch ($file) {
				case 'k2_configuration':
					$this->saveConfig($folder);
				break;
				case 'k2_user_groups':
					$this->saveUserGroups($folder);
				break;
				case 'k2_users':
					$this->saveUsers($folder);
				break;
				case 'k2_categories':
					$this->saveCategories($folder);
				break;
				case 'k2_items':
					$this->saveItems($folder);
				break;
				case 'k2_tags':
					$this->saveTags($folder);
				break;
				case 'k2_tags_xref':
					$this->saveTagsRefs($folder);
				break;
				case 'k2_rating':
					$this->saveRating($folder);
				break;
				case 'k2_comments':
					$this->saveComments($folder);
				break;
			}
		}
	}
	
	private function saveConfig($folder) {
		$db 	= JFactory::getDbo();
		$query	= $db->getQuery(true);
		
		$this->fileName = 'k2_configuration';
		$this->buildPath($folder);
		$this->loadXML();

		foreach ($this->xmlData->item as $item) {
			$item = $this->prepareData($item);
			
			$query->clear();
			
			$query	->update($db->qn('#__extensions'))
					->set(array(
						$db->qn('enabled') . ' = ' . $db->q($item['enabled']),
						$db->qn('params') . ' = ' . $db->q($item['params'])
					))
					->where($db->qn('type') . ' = ' . $db->q($item['type']) . ' AND ' . $db->qn('element') . ' = ' . $db->q($item['element']));
			$db->setQuery($query);
			$db->execute();
		}
	}
	
	private function saveUserGroups($folder) {
		$this->fileName = 'k2_user_groups';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {
			$table = $this->getTable('K2UserGroup');
			
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$table->bind($item);
				$table->id			= null;
				$table->name		= $item['name'];
				$table->permissions	= $item['permissions'];
				
				if ($table->check()) {
					$table->store();
					$this->tableData[] = $table;
				}
			}
		}
	}
	
	private function saveUsers($folder) {
		$this->fileName = 'k2_users';
		$this->buildPath($folder);
		$this->loadXML();
		
		$table	= $this->getTable('K2User');
			
		foreach ($this->xmlData->item as $item) {
			$item		= $this->prepareData($item);
			$user_id	= $this->getUserId($item['userName'], false);
			
			$table->bind($item);
			$table->id			= null;
			$table->userID		= $user_id;
			$table->userName	= $item['userName'];
			$table->gender		= $item['gender'];
			$table->description	= $item['description'];
			$table->url			= $item['url'];
			$table->group		= $this->getUserGroupId($item['group']);
			$table->notes		= $item['notes'];
			
			if ($table->check()) {
				$table->store();
				$this->tableData[] = $table;
			}
		}
		
		// Add user image to media folder
		jimport('joomla.filesystem.file');
		
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select('*')
				->from($db->qn('#__k2_users'));
		$db->setQuery($query);
		$users = $db->loadObjectList();
		
		foreach ($users as $user) {
			$image_template	= JPATH_SITE . '/templates/rstrone/images/sampledata/k2/users/' . $this->getUserName($user->userID) . '.jpg';
			$image_media	= JPATH_SITE . '/media/k2/users/' . $user->id . '.jpg';
			
			if (JFile::exists($image_template)) {
				$query->clear();
				
				$query	->update($db->qn('#__k2_users'))
						->set($db->qn('image') . ' = ' . $db->q($user->id . '.jpg'))
						->where($db->qn('id') . ' = ' . $db->q($user->id));
				$db->setQuery($query);
				$db->execute();
				
				JFile::copy($image_template, $image_media);
			}
		}
	}
	
	private function saveCategories($folder) {
		$this->fileName = 'k2_categories';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {
			$table = $this->getTable('K2Category');
			
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$table->bind($item);
				$table->id			= null;
				$table->name		= $item['name'];
				$table->alias		= $item['alias'];
				$table->description	= $item['description'];
				$table->parent		= $this->getCategoryId($item['parent']);
				$table->image		= $item['image'];
				$table->params		= $item['params'];
				
				if ($table->check()) {
					$table->store();
					$this->tableData[] = $table;
				}
			}
		}
	}
	
	private function saveItems($folder) {
		$this->fileName = 'k2_items';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {
			$table = $this->getTable('K2Item');
			
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$table->bind($item);
				$table->id				= null;
				$table->title			= $item['title'];
				$table->alias			= $item['alias'];
				$table->catid			= $this->getCategoryId($item['catid']);
				$table->introtext		= $item['introtext'];
				$table->fulltext		= $item['fulltext'];
				$table->created			= $item['created'];
				$table->created_by		= $this->getUserId($item['created_by'], true);
				$table->publish_up		= $item['publish_up'];
				$table->featured		= $item['featured'];
				$table->image_caption	= $item['image_caption'];
				$table->image_credits	= $item['image_credits'];
				$table->video_caption	= $item['video_caption'];
				$table->video_credits	= $item['video_credits'];
				$table->hits			= $item['hits'];
				$table->params			= $item['params'];
				
				if ($table->check()) {
					$table->store();
					$this->tableData[] = $table;
				}
			}
		}
		
		// Add item images to media folder
		jimport('joomla.filesystem.file');
		
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select('*')
				->from($db->qn('#__k2_items'));
		$db->setQuery($query);
		$items = $db->loadObjectList();
		
		foreach ($items as $item) {
			$images_template_path	= JPATH_SITE . '/templates/rstrone/images/sampledata/k2/items/';
			$images_media_path		= JPATH_SITE . '/media/k2/items/';
			
			if (JFolder::exists($images_template_path . $item->alias)) {
				$images = JFolder::files(JPATH_SITE . '/templates/rstrone/images/sampledata/k2/items/' . $item->alias);
				
				if(count($images)) {
					foreach ($images as $image) {
						$image			= JFile::stripExt($image);
						$image_template	= $images_template_path . $item->alias . '/' . $image . '.jpg';
						
						if (strpos($image, '_') > 0) {
							$sizes			= explode('_', $image);
							$image_media	= $images_media_path . 'cache/' . md5('Image' . $item->id) . '_' . $sizes[1] . '.jpg';
						} else {
							$image_media	= $images_media_path . 'src/' . md5('Image' . $item->id) . '.jpg';
						}
						JFile::copy($image_template, $image_media);
					}
				}
			}
		}
	}
	
	private function saveTags($folder) {
		$this->fileName = 'k2_tags';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {
			$table = $this->getTable('K2Tag');
			
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$table->bind($item);
				$table->id			= null;
				$table->name		= $item['name'];
				$table->published	= 1;
				
				if ($table->check()) {
					$table->store();
					$this->tableData[] = $table;
				}
			}
		}
	}
	
	private function saveTagsRefs($folder) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$this->fileName = 'k2_tags_xref';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {		
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$query->clear();
				$query	->insert($db->qn('#__k2_tags_xref'))
						->columns(array(
							$db->qn('tagID'),
							$db->qn('itemID')
						))
						->values(
							$db->q($this->getTagId($item['tagID'])) . ', ' .
							$db->q($this->getItemId($item['itemID']))
						);
				$db->setQuery($query);
				$db->execute();
			}
		}
	}
	
	private function saveRating($folder) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$this->fileName = 'k2_rating';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {		
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$query->clear();
				$query	->insert($db->qn('#__k2_rating'))
						->columns(array(
							$db->qn('itemID'),
							$db->qn('rating_sum'),
							$db->qn('rating_count')
						))
						->values(
							$db->q($this->getItemId($item['itemID'])) . ', ' .
							$db->q($item['rating_sum']) . ', ' .
							$db->q($item['rating_count'])
						);
				$db->setQuery($query);
				$db->execute();
			}
		}
	}
	
	private function saveComments($folder) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$this->fileName = 'k2_comments';
		$this->buildPath($folder);
		$this->loadXML();
		$items = XMLParser::getItems();

		if (count($items)) {		
			foreach ($items as $item) {
				$item = $this->prepareData($item);
				
				$query->clear();
				$query	->insert($db->qn('#__k2_comments'))
						->columns(array(
							$db->qn('itemID'),
							$db->qn('userID'),
							$db->qn('userName'),
							$db->qn('commentDate'),
							$db->qn('commentText'),
							$db->qn('commentEmail'),
							$db->qn('commentURL'),
							$db->qn('published')
						))
						->values(
							$db->q($this->getItemId($item['itemID'])) . ', ' .
							$db->q($this->getUserId($item['userID'], true)) . ', ' .
							$db->q($item['userName']) . ', ' .
							$db->q($item['commentDate']) . ', ' .
							$db->q($item['commentText']) . ', ' .
							$db->q($item['commentEmail']) . ', ' .
							$db->q($item['commentURL']) . ', ' .
							$db->q($item['published'])
						);
				$db->setQuery($query);
				$db->execute();
			}
		}
	}
	
	private function getCategoryId($alias) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select($db->qn('id'))
				->from($db->qn('#__k2_categories'))
				->where($db->qn('alias') . ' = ' . $db->q($alias));
		$db->setQuery($query);
		$id = $db->loadResult();
		
		return $id;
	}
	
	public function getUserId($ref, $unique) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		if ($unique) {
			$query	->select($db->qn('id'))
					->from($db->qn('#__users'))
					->where($db->qn('username') . ' = ' . $db->q($ref));
			$db->setQuery($query);
		} else {
			$query	->select($db->qn('id'))
					->from($db->qn('#__users'))
					->where($db->qn('name') . ' = ' . $db->q($ref));
			$db->setQuery($query);
		}
		
		$user_id = $db->loadResult();
		
		return $user_id;
	}
	
	private function getUserGroupId($group_name) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select($db->qn('id'))
				->from($db->qn('#__k2_user_groups'))
				->where($db->qn('name') . ' = ' . $db->q($group_name));
		$db->setQuery($query);
		$user_group_id = $db->loadResult();
		
		return $user_group_id;
	}
	
	private function getUserName($id) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select($db->qn('username'))
				->from($db->qn('#__users'))
				->where($db->qn('id') . ' = ' . $db->q($id));
		$db->setQuery($query);
		
		$user_name = $db->loadResult();
		
		return $user_name;
	}

	private function getTagId($name) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select($db->qn('id'))
				->from($db->qn('#__k2_tags'))
				->where($db->qn('name') . ' = ' . $db->q($name));
		$db->setQuery($query);
		$id = $db->loadResult();
		
		return $id;
	}
	
	private function getItemId($alias) {
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		
		$query	->select($db->qn('id'))
				->from($db->qn('#__k2_items'))
				->where($db->qn('alias') . ' = ' . $db->q($alias));
		$db->setQuery($query);
		$id = $db->loadResult();
		
		return $id;
	}
}
