<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_menus/tables/');

/**
 * Class XMLParserMenu
 */
class XMLParserMenu extends XMLParser
{
	/**
	 * @return mixed
	 */
	protected function getTable()
	{
		return JTable::getInstance('Menu', 'MenusTable');
	}

	/**
	 * @param     $item
	 * @param     $folder
	 * @param int $parent_id
	 *
	 * @return bool
	 */
	public function saveItem($item, $folder, $parent_id = 1)
	{
		$item     = $this->prepareData($item);
		$children = $this->getItemChildren($item);

		$component_id = 0;
		if ($item['type'] == 'component')
		{
			$component_id = $this->getComponentID($item['link']);

			if (!$component_id)
			{
				return false;
			}
		}

		if (substr($item['link'], 0, 1) != '#')
		{
			$link = $this->buildNewLink($item['link'], $folder);
		}
		else
		{
			$link = $item['link'];
		}
		$ref = $item['ref'];
		unset($item['ref']);
		
		$templateid = $item['templateid'];
		unset($item['templateid']);

		$table 					= $this->getTable();
		$table->bind($item);
		$table->id        		= null;
		$table->link      		= $link;
		$table->published 		= $this->setDefaultValue($item, 'published', 1);;
		$table->component_id	= $component_id;
		$table->access       	= 1;
		$table->level        	= $this->setDefaultValue($item, 'level', 0);
		$table->home         	= $this->setDefaultValue($item, 'home', 0);
		$table->client_id    	= $this->setDefaultValue($item, 'client_id', 0);
		$table->language     	= '*';
		$table->params       	= $this->parseParams($item['params'], $folder);

		$id = XMLParser::getInstance('Templates', $folder)->getTemplateId($templateid);
		if ((bool) $id)
		{
			$table->template_style_id = $id;
		}
		
		$table->setLocation($parent_id, 'last-child');

		if ($table->check())
		{
			if ($existingId = $this->checkIfExists($table, $parent_id))
			{
				$table->id = $existingId;
			}
			else
			{
				$table->store();
				$table->rebuildPath($table->id);
				$table->rebuild($table->id, $table->lft, $table->level, $table->path);
			}

			$this->tableData[$ref] = $table;
			if ($children)
			{
				foreach ($children as $child)
				{
					$this->saveItem($child, $folder, $table->id);
				}
			}
		}
	}

	/**
	 * @param $params
	 * @param $folder
	 *
	 * @return mixed|string
	 */
	private function parseParams($params, $folder)
	{
		$data = json_decode($params);

		// for Menu Item Alias: replace "aliasoptions" param with the corresponding "idmenuX"
		if (isset($data->aliasoptions))
		{
			$data->aliasoptions = $this->getMenuId($data->aliasoptions);
		}
		if (isset($data->featured_categories) && $data->featured_categories != '')
		{
			if (is_array($data->featured_categories))
			{
				foreach ($data->featured_categories as &$cat)
				{
					if ($cat != '')
					{
						$infoCat = $this->getAliasExtension($cat);
						$cat     = XMLParser::getInstance('Categories', $folder)->getCategoryId($infoCat->alias, $infoCat->extension);
					}
				}
			}
			else
			{
				$infoCat                   = $this->getAliasExtension($data->featured_categories);
				$data->featured_categories = XMLParser::getInstance('Categories', $folder)->getCategoryId($infoCat->alias, $infoCat->extension);
			}

		}
		$data = json_encode($data);

		return $data;
	}

	/**
	 * @param $cat
	 *
	 * @return stdClass
	 */
	private function getAliasExtension($cat)
	{
		$data            = new stdClass;
		$expl            = explode('|', $cat);
		$data->alias     = $expl[0];
		$data->extension = ($expl[1] == '' ? 'com_content' : $expl[1]);

		return $data;
	}

	/**
	 * @param $table
	 * @param $parent_id
	 *
	 * @return mixed
	 */
	protected function checkIfExists($table, $parent_id)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);

		$query->select($db->qn('id'))
			->from($db->qn('#__menu'))
			->where($db->qn('client_id') . ' = ' . $db->q($table->client_id))
			->where($db->qn('parent_id') . ' = ' . $db->q($parent_id))
			->where($db->qn('alias') . ' = ' . $db->q($table->alias))
			->where($db->qn('language') . ' = ' . $db->q($table->language));

		return $db->setQuery($query)->loadResult();
	}

	/**
	 * @param $url
	 *
	 * @return bool
	 */
	protected function getComponentId($url)
	{
		// Parse the given URL
		$parts = parse_url($url);

		// Must have a query
		if (isset($parts['query']))
		{
			parse_str($parts['query'], $query);

			// Get the ?option from the URL
			if (isset($query['option']))
			{
				$component = JComponentHelper::getComponent($query['option']);
				if (isset($component->id))
				{
					return $component->id;
				}
			}
		}

		return false;
	}

	/**
	 * @param $ref
	 *
	 * @return string
	 */
	public function getMenuId($ref)
	{
		return isset($this->tableData[$ref]) ? $this->tableData[$ref]->id : '';
	}
}
