<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

/**
 * Class XMLParserModules
 */
class XMLParserModules extends XMLParser
{
	/**
	 * @return mixed
	 */
	protected function getTable()
	{
		JTable::addIncludePath(JPATH_SITE . '/libraries/legacy/table/menu/');

		return JTable::getInstance('Module', 'JTable');
	}

	/**
	 * @param $item
	 * @param $folder
	 */
	public function saveItem($item, $folder)
	{
		$item    = $this->prepareData($item);
		$setMenu = false;
		if (isset($item['menuid']))
		{
			$setMenu = json_decode($item['menuid']);
			unset($item['menuid']);
		}

		$table = $this->getTable();
		$table->bind($item);
		$table->id        = null;
		$table->content   = $this->buildNewLinks($item['content'], $folder);
		$table->access    = $this->setDefaultValue($item, 'access', 1);
		$table->showtitle = $this->setDefaultValue($item, 'showtitle', 'showtitle');
		$table->ordering  = $this->setDefaultValue($item, 'ordering', 1);
		$table->published = 1;
		$table->params    = $this->parseParams($item['params'], $folder);
		$table->language  = '*';

		if ($table->check())
		{
			$table->store();
			$this->tableData[] = $table;

			if ($setMenu)
			{
				foreach ($setMenu->ids as $sMenu)
				{
					$dateReferinta = $this->checkMenuSign($sMenu);

					$ref  = $dateReferinta->value;
					$sign = $dateReferinta->sign;

					$menuid = XMLParser::getInstance('Menu', $folder)->getMenuId($ref);

					if (empty($menuid))
					{
						continue;
					}
					$this->addToModulesMenu(array(
						'moduleid' => $table->id,
						'menuid'   => (int) $menuid,
						'sign'     => $sign
					));
				}
			}
			else
			{
				$this->addToModulesMenu(array(
					'moduleid' => $table->id,
					'menuid'   => 0,
					'sign'     => ''
				));
			}
		}

	}

	/**
	 * @param $params
	 * @param $folder
	 *
	 * @return mixed|string
	 */
	private function parseParams($params, $folder)
	{
		$data = json_decode($params);
		if (isset($data->catid) && $data->catid != '')
		{
			if (is_array($data->catid))
			{
				foreach ($data->catid as &$cat)
				{
					if ($cat != '')
					{
						$infoCat = $this->getAliasExtension($cat);
						$cat     = XMLParser::getInstance('Categories', $folder)->getCategoryId($infoCat->alias, $infoCat->extension);
					}
				}
			}
			else
			{
				$infoCat     = $this->getAliasExtension($data->catid);
				$data->catid = XMLParser::getInstance('Categories', $folder)->getCategoryId($infoCat->alias, $infoCat->extension);
			}

		}
		$data = json_encode($data);

		return $data;
	}

	/**
	 * @param $cat
	 *
	 * @return stdClass
	 */
	private function getAliasExtension($cat)
	{
		$data            = new stdClass;
		$expl            = explode('|', $cat);
		$data->alias     = $expl[0];
		$data->extension = isset($expl[1]) ? $expl[1] : 'com_content';

		return $data;
	}

	/**
	 * @param $string
	 *
	 * @return stdClass
	 */
	private function checkMenuSign($string)
	{
		$menu = new stdClass;
		if (substr($string, 0, 1) != '-')
		{
			$menu->value = $string;
			$menu->sign  = '';
		}
		else
		{
			$menu->value = substr($string, 1);
			$menu->sign  = '-';
		}

		return $menu;
	}

	/**
	 * @param $data
	 */
	private function addToModulesMenu($data)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->clear()
			->insert('#__modules_menu')
			->columns(array($db->quoteName('moduleid'), $db->quoteName('menuid')))
			->values((int) $data['moduleid'] . ',' . $data['sign'] . $data['menuid']);
		$db->setQuery($query);
		$db->execute();
	}

	/**
	 * @param $alias
	 *
	 * @return mixed
	 */
	public function getModuleId($alias)
	{
		static $cache = array();

		if (!isset($cache[$alias]))
		{
			foreach ($this->tableData as $position => $table)
			{
				if ($table->alias == $alias)
				{
					$cache[$alias] = $position;
					break;
				}
			}
		}

		return $this->tableData[$cache[$alias]]->id;
	}
}
