<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_rsmediagallery/tables/');

/**
 * Class XMLParserRSMediaGalleryItems
 */
class XMLParserRSMediaGalleryItems extends XMLParser
{
	/**
	 * @var
	 */
	public $error;

	/**
	 * @return mixed
	 */
	protected function getTable()
	{
		return JTable::getInstance('Items', 'RSMediaGalleryTable');
	}

	/**
	 * @param $item
	 * @param $folder
	 */
	public function saveItem($item, $folder)
	{
		$check = XMLParser::checkIfInstalled('com_rsmediagallery');

		if ($check)
		{
			$item        = $this->prepareData($item);
			$file        = $item['original_filename'];
			$tag         = $this->setDefaultValue($item, 'tag', null);
			$title       = $this->setDefaultValue($item, 'title', null);
			$description = $this->setDefaultValue($item, 'description', '');
			$url         = $this->setDefaultValue($item, 'url', '');
			$params      = $this->setDefaultValue($item, 'params', '');
			$this->copyImage($file, $params, $title, $description, $tag, $url);
		}
	}

	/**
	 * @param        $file
	 * @param        $params
	 * @param null   $title
	 * @param string $description
	 * @param null   $tag
	 * @param string $url
	 */
	private function copyImage($file, $params, $title = null, $description = '', $tag = null, $url = '')
	{
		$template        = self::getTemplateName();
		$from_path       = JPATH_SITE . '/templates/' . $template . '/images/sampledata/rsmediagallery/' . $file;
		$from_thumbs_path = JPATH_SITE . '/templates/' . $template . '/images/sampledata/rsmediagallery/thumbs/' . $file;

		$fileName        = explode('.', $file);
		$ext             = $fileName[1];
		$fileName        = $fileName[0];
		$hash            = md5(uniqid($fileName));
		$upload_location = JPATH_SITE . '/components/com_rsmediagallery/assets/gallery';

		jimport('joomla.filesystem.file');
		JFile::copy($from_path, $upload_location . '/original/' . $hash . '.' . $ext);

		$componentParams = JComponentHelper::getParams('com_rsmediagallery');
		$perms           = octdec('0' . $componentParams->get('file_perms', '644'));
		@chmod($upload_location . '/original/' . $hash . '.' . $ext, $perms);

		JFile::copy($from_thumbs_path, $upload_location . '/' . $hash . '.' . $ext);
		// set file permissions
		@chmod($upload_location . '/' . $hash . '.' . $ext, $perms);

		// get date object
		$date = JFactory::getDate();

		// save to db
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_rsmediagallery/tables');
		$image                    = $this->getTable();
		$image->original_filename = $fileName . '.' . $ext;
		$image->filename          = $hash . '.' . $ext;
		$image->title             = ($title == null ? JFile::stripExt($fileName) : $title);
		$image->description       = $description;
		$image->type              = 'image';
		$image->url               = $url;
		// set correct ordering
		$image->ordering = $image->getNextOrder();
		$image->params   = $params;

		$image->created  = $date->toSql();
		$image->modified = $date->toSql();

		// set it to unpublished
		$image->published = 1;
		if ($image->store())
		{
			// add tags
			if ($tag != null)
			{
				$tags = explode(',', $tag);
				foreach ($tags as $curr_tag)
				{
					$this->addToRSmediagalleryTags(array(
						'item_id' => $image->id,
						'tag'     => $curr_tag
					));
				}
			}
		}
	}

	/**
	 * @param $data
	 */
	private function addToRSmediagalleryTags($data)
	{
		$db    = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->clear()
			->insert('#__rsmediagallery_tags')
			->columns(array($db->quoteName('item_id'), $db->quoteName('tag')))
			->values((int) $data['item_id'] . ",'" . $data['tag'] . "'");
		$db->setQuery($query);
		$db->execute();
	}

	/**
	 * @param $alias
	 *
	 * @return mixed
	 */
	public function getGalleryItemId($alias)
	{
		static $cache = array();

		if (!isset($cache[$alias]))
		{
			foreach ($this->tableData as $position => $table)
			{
				if ($table->alias == $alias)
				{
					$cache[$alias] = $position;
					break;
				}
			}
		}

		return $this->tableData[$cache[$alias]]->id;
	}
}
