<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die('Restricted access');

/**
 * Class XMLParser
 */
class XMLParser
{
	/**
	 * @var
	 */
	protected $fileName;
	/**
	 * @var
	 */
	protected $path;
	/**
	 * @var
	 */
	protected $xmlData;
	/**
	 * @var
	 */
	protected $tableData;

	/**
	 * XMLParser constructor.
	 *
	 * @param $path
	 */
	public function __construct($path)
	{
		$this->buildFileName();
		$this->buildPath($path);
		if ($this->fileName != 'k2') {
			$this->loadXML();
		}
	}

	/**
	 *
	 */
	protected function buildFileName()
	{
		$className      = get_class($this);
		$this->fileName = strtolower(substr($className, strlen('XMLParser')));
	}

	/**
	 * @param $path
	 */
	protected function buildPath($path)
	{
		$this->path = dirname(__FILE__) . '/sampledata/' . $path . '/';
	}

	/**
	 *
	 */
	protected function loadXML()
	{
		$this->xmlData = simplexml_load_file($this->path . $this->fileName . '.xml');
	}

	/**
	 * @param $childName
	 * @param $folder
	 *
	 * @return mixed
	 */
	public static function getInstance($childName, $folder)
	{
		static $instances = array();

		$childName = strtolower($childName);
		if (!isset($instances[$childName]))
		{
			if (substr($childName, 0, 2) == 'k2') {
				$childName = 'k2';
			}
			
			$className = 'XMLParser' . $childName;

			if (!class_exists($className))
			{
				require_once dirname(__FILE__) . '/parsers/' . $folder . '/' . $childName . '.php';
			}

			$instances[$childName] = new $className($folder);
		}

		return $instances[$childName];
	}

	/**
	 * @return array
	 */
	public function getItems()
	{
		$data = array();
		foreach ($this->xmlData->item as $item)
		{
			$data[] = $item;
		}

		return $data;
	}

	/**
	 * @param $item
	 *
	 * @return array
	 */
	protected function prepareData($item)
	{
		$data = array();
		foreach ($item as $key => $val)
		{
			if ($key != 'child')
			{
				$data[$key] = (string) $val;
			}
			else
			{
				$data[$key][] = $val;
			}
		}

		return $data;
	}

	/**
	 * @param $item
	 *
	 * @return array
	 */
	protected function getItemChildren($item)
	{
		$children = array();
		if ($item && isset($item['child']))
		{
			foreach ($item['child'] as $child)
			{
				$children[] = $child;
			}
		}

		return $children;
	}

	/**
	 * @param     $item
	 * @param     $key
	 * @param     $value
	 * @param int $unset
	 *
	 * @return mixed
	 */
	protected function setDefaultValue($item, $key, $value, $unset = 0)
	{
		if (isset($item[$key]))
		{
			return $item[$key];
			if ($unset)
			{
				unset($item[$key]);
			}
		}
		else
		{
			return $value;
		}
	}

	/**
	 * @return mixed
	 */
	public static function getTemplateName()
	{
		$ds      = preg_quote(DIRECTORY_SEPARATOR);
		$pattern = '#' . $ds . 'templates' . $ds . '([^' . $ds . ']+)' . $ds . 'classes' . $ds . '#is';

		if (preg_match($pattern, dirname(__FILE__), $match))
		{
			return $match[1];
		}
	}

	/**
	 * @param $extension
	 *
	 * @return bool
	 */
	public static function checkIfInstalled($extension)
	{
		$table = JTable::getInstance('Extension', 'JTable');
		$table->load(array('element' => $extension));

		return !is_null($table->extension_id);
	}

	/**
	 * @param $url
	 * @param $folder
	 *
	 * @return string
	 */
	protected function buildNewLink($url, $folder)
	{
		// Parse the given url
		$parts = parse_url($url);
		parse_str($parts['query'], $query);

		// Can't continue without a component
		if (empty($query['option']))
		{
			return $url;
		}

		if (empty($query['view']))
		{
			return $url;
		}

		if (!empty($query['Itemid']))
		{
			$Itemid = XMLParser::getInstance('Menu', $folder)->getMenuId($query['Itemid']);
			if (!empty($Itemid))
			{
				$query['Itemid'] = $Itemid;
			}
			else
			{
				unset($query['Itemid']);
			}
		}

		switch ($query['view'])
		{
			case 'category':
				$query['id'] = XMLParser::getInstance('Categories', $folder)->getCategoryId($query['id'], $query['option']);
				break;
			case 'contact':
				$query['id'] = XMLParser::getInstance('ContactDetails', $folder)->getContactId($query['id']);
				break;
			case 'article':
				$query['id'] = XMLParser::getInstance('Content', $folder)->getContentId($query['id']);
				break;
			case 'newsfeed':
				$query['id'] = XMLParser::getInstance('Newsfeeds', $folder)->getNewsfeedId($query['id']);
				break;
			case 'page':
				$query['id'] = XMLParser::getInstance('RSPageBuilder', $folder)->getPageId($query['id']);
				break;
			case 'itemlist':
				if ($query['layout'] == 'user') {
					$query['id'] = XMLParser::getInstance('K2', $folder)->getUserId($query['id'], true);
				}
				break;
		}

		$url = 'index.php?' . http_build_query($query);
		if (!empty($parts['fragment']))
		{
			$url .= '#' . $parts['fragment'];
		}

		return $url;
	}

	/**
	 * @param $content
	 * @param $folder
	 *
	 * @return mixed
	 */
	protected function buildNewLinks($content, $folder)
	{
		$replace = array();
		$with    = array();

		$pattern = '#href="(index\.php\?option\=.*?)"#i';

		if (preg_match_all($pattern, $content, $matches))
		{
			foreach ($matches[1] as $i => $link)
			{
				$newlink = $this->buildNewLink($link, $folder);
				if ($newlink != $link)
				{
					$replace[] = $link;
					$with[]    = $newlink;
				}
			}
		}

		if ($replace)
		{
			$content = str_replace($replace, $with, $content);
		}

		return $content;
	}
}
