<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Class RSTemplateRowSettings
 */
class RSTemplateRowSettings
{
	/**
	 * @var JRegistry
	 */
	protected $data;
	/**
	 * @var
	 */
	protected $position;

	/**
	 * Helper function for escaping strings
	 * @method escape
	 *
	 * @param  string $string
	 *
	 * @return string
	 */
	public function escape($string)
	{
		return htmlentities($string, ENT_COMPAT, 'UTF-8');
	}

	/**
	 * Creates a new JRegistry Object
	 * @method __construct
	 *
	 * @param  object $data
	 */
	public function __construct($data)
	{
		$this->data = new JRegistry($data);
	}

	/**
	 * Sets the position we need to change
	 * @method setPosition
	 *
	 * @param  string $position
	 */
	public function setPosition($position)
	{
		$this->position = $position;
	}

	/**
	 * Shortcut function for __get
	 * @method get
	 *
	 * @param  string $name
	 * @param  string $default
	 *
	 * @return mixed
	 */
	public function get($name, $default = null)
	{
		return $this->data->get($this->position . '.' . $name, $default);
	}

	/**
	 * Used to grab the actual settings in the frontend
	 * @method __get
	 *
	 * @param  string $name
	 *
	 * @return mixed
	 */
	public function __get($name)
	{
		switch (strtolower($name))
		{
			case 'isfullwidth':
				return (bool) $this->get('general.fw', false);
				break;
			case 'rowclass':
				$class = ($this->get('general.cls') == '') ? '' : ' '.$this->get('general.cls');
				
				return $class;
				break;
			case 'columns':
				$columns = $this->get('cols', (object) array());

				$animation = array(
					'r' => 'enter right',
					'l' => 'enter left',
					'd' => 'enter top',
					'u' => 'enter bottom',
					'f' => 'move 0px',
					0   => false
				);

				$widget = array(
					0 => false,
					1 => 'Social',
					2 => 'Twitter',
                    3 => 'Copyright'
				);

				$parsed = array();
				foreach ($columns as $name => $val)
				{
					$wgdts   = explode(',', $val->wg);
					$widgets = array();
					foreach ($wgdts as $wgdt)
					{
						if ($wgdt == '0')
						{
							continue;
						}

						$widgets[] = $widget[$wgdt];
					}

					$parsed[$name] = (object) array(
						'name'      => $name,
						'isremoved' => $val->size == 'rem',
						'size'      => $val->size,
						'animation' => $animation[$val->an],
						'widget'    => $widgets,
					);
				}

				return $parsed;
				break;
			case 'css':
				$properties = array(
					'background-color' => $this->get('css.bg-color'),
					'margin-top'       => $this->get('css.m-top'),
					'margin-bottom'    => $this->get('css.m-bot'),
					'padding-top'      => $this->get('css.p-top'),
					'padding-bottom'   => $this->get('css.p-bot'),
				);
				$style      = array();
				foreach ($properties as $property => $value)
				{

					if (strlen($value))
					{
						$style[] = $property . ':' . $this->escape($value) . ';';
					}

				}

				if ($style)
				{
					return implode(' ', $style);
				}

				return false;
				break;

		}

		return true;
	}

}
