<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;
require_once dirname(__FILE__) . '/helper.php';
require_once dirname(__FILE__) . '/widget.php';
require_once dirname(__FILE__) . '/rowsettings.php';
require_once dirname(__FILE__) . '/megamenu.php';

/**
 * Class RSTemplate
 *
 * Returns the actual template object
 */
class RSTemplate
{
	/**
	 * Container for the widgets
	 *
	 * @var array
	 */
	public $widgets = array();

	/**
	 * Reference to the current document
	 *
	 * @var object
	 */
	protected $document;

	/**
	 * @var string
	 */
	protected $root;

	/**
	 * The active component
	 *
	 * @var string
	 */
	protected $option;

	/**
	 * Template settings
	 *
	 * @var null
	 */
	protected $params;

	/**
	 * Path to the custom CSS file
	 *
	 * @var string
	 */

	public $customCSSPath;

	/**
	 * Constructs the object with the parameters defined in the template options
	 *
	 * @method      __construct
	 *
	 * @param      $document
	 * @param null $params
	 *
	 * @throws Exception
	 */
	public function __construct($document, $params = null)
	{
        jimport('joomla.filesystem.folder');
		$this->root     = JUri::root(true);
		$this->option   = JFactory::getApplication()->input->getCmd('option');
		$this->document = &$document;
        $this->params   = $this->getTemplateStyleParams($this->getTemplateId());
		$widgets        = JFolder::folders(RSTEMPLATE_PATH . '/widgets/');

		foreach ($widgets as $widget)
		{
			$this->widgets[$widget] = new RSTemplateWidget($widget, $this->params);
		}

        if (!preg_match('/<a href=\"https:\/\/www.rsjoomla.com\/joomla-templates.html\" title=\"Responsive Joomla Templates\" target=\"_blank\">Responsive Joomla Templates<\/a> by RSJoomla!/', file_get_contents(RSTEMPLATE_PATH . '/index.php'))) {
            die;
        }
	}

	/**
	 * Initiate the template
	 * @method initHead
	 */
	public function initHead()
	{
		// Load bootstrap
		if ($this->params->get('load_bootstrap'))
		{
			$this->addBootstrap();
		}
		// Load jQuery
		$this->addJQuery();

		// Load Icons
		$this->addCSS('icons.css');

        // Load Preloader
        $this->addJS('jquery/jquery.animsition.min.js');
        $this->addCSS('animsition.min.css');

		// Add Google Font
		$this->addGoogleFont();

		// Load Google Analytics
		$this->addGoogleAnalytics();

		// Add additional header content
		$this->addHeaderContent();

		// Conditional IE8 / IE9
		$this->conditionalStyle();

		// Load component and module css
		$this->addComponentCSS();
		$this->addModulesCSS();

        // Load template styles
        $this->addTemplateCSS();

        // Load template.js
        $this->addTemplateJs();

		// Load custom.css
		$this->addCustomCSS();

		// Load custom.js
		$this->addCustomJS();
	}

	/**
	 * Each row has its own settings, we create instances for each one
	 *
	 * @method getRowSettings
	 *
	 * @param  string $position
	 *
	 * @return object
	 */
	public function getRowSettings($position)
	{
		static $instance = array();

		if (!array_key_exists($position, $instance))
		{
			$instance[$position] = new RSTemplateRowSettings($this->params->get('defined_row_settings'));
			$instance[$position]->setPosition($position);
		}

		return $instance[$position];
	}

	/**
	 * Returns all positions with its own settings
	 * @method getPositions
	 *
	 * @return object
	 */
	public function getPositions()
	{

		if ($positions = $this->params->get('defined_positions', 'top,after-top,main-menu,slideshow,featured,breadcrumbs,before-higher,higher,after-higher,before-content,all-content,after-content,before-middle,middle,after-middle, before-lower,lower,after-lower,before-bottom,bottom,after-bottom,before-footer,footer,after-footer,copyright'))
		{
			$thePositions = array();
			$positions    = explode(',', $positions);

			foreach ($positions as $i => $position)
			{
				$file = realpath(JPATH_SITE . '/templates/' . $this->document->template . '/positions/' . $position . '.php');

				if (!is_file($file))
				{
					$file = realpath(JPATH_SITE . '/templates/' . $this->document->template . '/positions/default.php');
				};
				$thePositions[$position] = (object) array('name' => $position, 'settings' => $this->getRowSettings($position), 'file' => $file,);
			}

			return $thePositions;
		}

		return false;

	}

	/**
	 * Grab the component active on the page
	 * @method getOption
	 *
	 * @return string
	 */
	public function getOption()
	{
		return $this->option;
	}

	/**
	 * Echoes the conditional style for IE8 and IE9
	 * @method conditionalStyle
	 *
	 * @return string
	 */
	public function conditionalStyle()
	{
		$html = "\n" . '
	<!--[if IE8]>
		<link rel="stylesheet" type="text/css" href="' . $this->document->baseurl . '/templates/' . $this->document->template . '/css/ie8.css" />
		<script src="' . $this->document->baseurl . $this->getJSPath("html5", "html5.js") . '"></script>
	<![endif]-->
	<!--[if IE 9]>
		<link rel="stylesheet" type="text/css" href="' . $this->document->baseurl . '/templates/' . $this->document->template . '/css/ie9.css" />
		<script src="' . $this->document->baseurl . $this->getJSPath("html5", "html5.js") . '"></script>
		<script src="' . $this->document->baseurl . '/templates/' . $this->document->template . '/js/jquery/jquery.placeholder.js"></script>
	<![endif]-->' . "\n";
		echo $html;
	}

	/**
	 * Returns the template ID
	 * @method getTemplateId
	 *
	 * @return int
	 */
	public static function getTemplateId()
	{
        $app = JFactory::getApplication();

        return (!empty($app->getMenu()->getActive()->template_style_id) ? (int) $app->getMenu()->getActive()->template_style_id : (int) $app->getTemplate(true)->id);
	}

	/**
	 * Returns the menu item ID
	 * @method getItemId
	 *
	 * @return int
	 */
	public function getItemId()
	{
		$app    = JFactory::getApplication();
		$menu   = $app->getMenu();
		$active = $menu->getActive();

		if ($active)
		{
			return (int) $active->id;
		}

		return $app->input->getInt('Itemid', 0);

	}

    public static function getTemplateStyleParams($style_id)
    {
        $registry 	= new JRegistry;
        $db 		= JFactory::getDbo();
        $query 		= $db->getQuery(true);

        $query->select($db->qn('params'))->from($db->qn('#__template_styles'))->where($db->qn('id').' = '.$db->q($style_id));
        $db->setQuery($query);
        $params 	= $db->loadResult();
        $params   	= $registry->loadString($params);

        return $params;
    }

	/**
	 * Check if the current page is the default homepage
	 * @method isHome
	 *
	 * @return boolean
	 */
	public function isHome()
	{
		$active = $this->getItemId();
		$app    = JFactory::getApplication();
		$menu   = $app->getMenu();
		$lang   = '*';

		if ($app->getLanguageFilter())
		{
			$lang = JFactory::getLanguage()->getTag();
		}

		$home = $menu->getDefault($lang);

		return ($active && $home && $active == $home->id);
	}

	/**
	 * Check if the homepage content is hidden
	 * @method isHomeHidden
	 *
	 * @return boolean
	 */
	public function isHomeHidden()
	{
		$isHome = $this->isHome();

		return ($this->params->get('hide_home') && $isHome);
	}

	/**
	 * Adds the google analytics traking code
	 * @method addGoogleAnalytics
	 */
	public function addGoogleAnalytics()
	{

		if ($this->params->get('google_analytics_code'))
		{
			$code = $this->params->get('google_analytics_code');
			$this->document->addScriptDeclaration('
			(function(i,s,o,g,r,a,m){
				i["GoogleAnalyticsObject"]=r;
				i[r]=i[r]||function(){
					(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();
					a=s.createElement(o),m=s.getElementsByTagName(o)[0];
					a.async=1;
					a.src=g;m.parentNode.insertBefore(a,m)
				})
				(window,document,"script","//www.google-analytics.com/analytics.js","ga");
				ga("create", "' . $this->escape($code) . '", "auto");
				ga("set", "anonymizeIp", true);
				ga("send", "pageview");
			');
		}

	}

	/**
	 * Adds Google Tag Manger code
	 * @method addGoogleTagManager
	 */
	public function addGoogleTagManager()
	{

		if ($this->params->get('google_tag_manager_code'))
		{
			echo $this->params->get('google_tag_manager_code');
		}

	}

	/**
	 * Adds Header Content
	 * @method addHeaderContent
	 */
	public function addHeaderContent()
	{

		if (trim($this->params->get('add_header_content')) != '')
		{
			echo $this->params->get('add_header_content');
		}

	}

	/**
	 * Adds a scroll to top button
	 * @method addScrollTop
	 */
	public function addScrollTop()
	{

		if ($this->params->get('scroll_top_button'))
		{
			echo '<div class="rstpl-go-top visible-desktop"><a href="javascript:void(0)" title="><span class="icon-angle-up"></span></a></div>';
		}

	}

	/**
	 * Create the search form so we can create it anywhere we need it
	 * @method createSearch
	 *
	 * @return string
	 */
	public static function createSearch($type)
	{
        $searchForm = '';

        if ($type =='k2_search') {
            jimport('joomla.application.module.helper');
            $module = JModuleHelper::getModule('mod_k2_tools', 'K2 Tools');
			
			$moduleParams = new JRegistry();
			$moduleParams->loadString($module->params);
			$moduleParams->set('module_usage', '6');
			
			$module->params = $moduleParams->toString();

            $searchForm = '<li class="rstpl-navbar-search">';
            $searchForm .= '<a class="open-search" href="#"><i class="icon-search"></i></a>';
            $searchForm .= JModuleHelper::renderModule($module);
            $searchForm .= '</li>';

        } else if ($type =='rspagebuilder_search') {
            jimport('joomla.application.module.helper');
            $module = JModuleHelper::getModule('mod_rspagebuilder_search', 'RSPageBuilder! Search');

            $searchForm = '<li class="rstpl-navbar-search">';
            $searchForm .= '<a class="open-search" href="#"><i class="icon-search"></i></a>';
            $searchForm .= JModuleHelper::renderModule($module);
            $searchForm .= '</li>';
        } else if ($type == 'joomla_search') {
            $searchForm = '<li class="rstpl-navbar-search">';
            $searchForm .= '<a class="open-search" href="#"><i class="icon-search"></i></a>';
            $searchForm .= '<form action="' . JRoute::_('index.php') . '" method="post" class="form-inline">';
            $searchForm .= '<label for="mod-search-searchword" class="element-invisible">' . JText::_('LIB_RSTEMPLATE_SEARCH_VALUE') . '</label>';
            $searchForm .= '<input name="searchword" id="mod-search-searchword" maxlength="200" class="inputbox search-bar" placeholder="' . JText::_('LIB_RSTEMPLATE_SEARCH_VALUE') . '" type="text">';
            $searchForm .= '<input name="task" value="search" type="hidden">';
            $searchForm .= '<input name="option" value="com_search" type="hidden">';
            $searchForm .= '<input name="Itemid" value="' . JFactory::getApplication()->input->get('Itemid') . '" type="hidden">';
            $searchForm .= '</form>';
            $searchForm .= '</li>';
        }

		return $searchForm;
	}

    /**
     * Create the search form so we can create it anywhere we need it
     * @method createLogin
     *
     * @return string
     */
    public static function createLogin()
    {
        jimport('joomla.application.module.helper');
        $module     = JModuleHelper::getModule('mod_login', 'Login');
        $loginForm  = '';

        $loginForm .= '<li class="rstpl-navbar-login">';
        $loginForm .= '<a class="open-login" href="#"><i class="icon-user"></i></a>';
        $loginForm .= JModuleHelper::renderModule($module);
        $loginForm .= '</li>';

        return $loginForm;
    }

	/**
	 * Gets class of page body
	 * @method getBodyClass
	 */
    public function getBodyClass()
    {
        return array(
            'id'                            => 'rstpl-pagewrap',
            'class'                         => 'body custom_' . $this->getItemId() . ' animsition',
            'data-animsition-in-class'      => $this->params->get('preloader_effect'),
            'data-animsition-out-class'     => str_replace('-in', '-out', $this->params->get('preloader_effect')),
            'data-animsition-loading-class' => 'animsition-' . $this->params->get('preloader_type') . '-loading'
        );
    }

	/**
	 * Loads the google fonts needed
	 * @method addGoogleFont
	 */
	public function addGoogleFont()
	{
		if (is_null($this->params->get('loadGoogleFonts')) || $this->params->get('loadGoogleFonts')) {
			$uri             = JUri::getInstance();
			$prefix_protocol = 'http';

			if ($uri->isSSL())
			{
				$prefix_protocol .= 's';
			}

			$sufix = '';

			if ($this->params->get('googleFontSubset') && $this->params->get('googleFontSubset') != '')
			{
				$sufix = '&amp;subset=' . $this->params->get('googleFontSubset');
			}

			$fonts = array(
				'title'   => $this->params->get('googleTitleFont'),
				'content' => $this->params->get('googleContentFont'),
				'navbar'  => $this->params->get('googleNavbarFont')
			);

			/**
			 * title/content/navbar may use the same font, filter the results to only load once
			 * foreach + array_keys is faster than array_unique
			 */
			$load = array();
			foreach ($fonts as $key => $val)
			{
				$load[$val] = true;
			}
			$load = array_keys($load);

			foreach ($load as $font)
			{
				$this->document->addCustomTag('<link href="' . $prefix_protocol . '://fonts.googleapis.com/css?family=' . $font . $sufix . '" rel="stylesheet" type="text/css" />');
			}

			if (trim($this->params->get('google_title_classes')) != '')
			{
				$extraTitleSelectors = trim($this->params->get('google_title_classes'), ',');
				$this->document->addStyleDeclaration(
					$extraTitleSelectors . "{
						font-family: " . current(explode(":", $this->params->get('googleTitleFont'))) . "
					}"
				);
			}

			// Content selectors with Google Font
			if (trim($this->params->get('google_content_classes')) != '')
			{
				$extraContentSelectors = trim($this->params->get('google_content_classes'), ',');
				$this->document->addStyleDeclaration(
					$extraContentSelectors . "{
						font-family: " . current(explode(":", $this->params->get('googleTitleFont'))) . "
					}"
				);
			}
		}
	}

	/**
	 * Helper function to escape strings
	 * @method escape
	 *
	 * @param  string $string
	 *
	 * @return string
	 */
	public function escape($string)
	{
		return htmlentities($string, ENT_COMPAT, 'utf-8');
	}

	/**
	 * Add Javascript files
	 * @method addJS
	 *
	 * @param  string  $url
	 * @param  boolean $useRoot
	 * @param  boolean $return
	 * @param  boolean $defer
	 *
	 * @return string
	 */
	public function addJS($url, $useRoot = true, $return = false, $defer = false)
	{
		$prefix = '';
		if ($useRoot)
		{
			$prefix = $this->root . '/templates/' . $this->document->template . '/js/';
		}

		if ($return)
		{
			return $prefix . $url;
		}
		else
		{
			$this->document->addScript($prefix . $url);
		}

		if ($defer)
		{
			$this->document->addScript($prefix . $url, "text/javascript", false, true);
		}

		return true;
	}

	/**
	 * Adds default template JS libraries
	 * @method addTemplateJs
	 */
	public function addTemplateJs()
	{
		// load template JS file
		$this->addJS('template.js');

		// load menu compatibility on tablets
		$this->addJs('jquery/jquery.mobile.custom.js');
		$this->addJs('device-dropdown.js');

		// load waypoints
		$this->addJS('jquery/jquery.waypoints.min.js');
		
		// load fill text
		$this->addJs('jquery/jquery.filltext.js');
		
		// load progress bars
		$this->addJs('velocity.min.js');

		// load script for onepage navigation
		if ($this->params->get('one_page'))
		{
			$this->addJs('onepage.js');
		}

		// load scrollReveal if needed
		if ($this->params->get('fade_content'))
		{
			$this->addJS('scrollReveal.min.js');
		}

		// load sticky menu js
		if ($this->params->get('sticky_menu'))
		{
			$this->addJS('sticky-menu.js');
		}
	}

	/**
	 * Add jQuery
	 * @method addjQuery
	 */
	public function addjQuery()
	{
		$value = $this->params->get('load_jquery', 1);

		if ((bool) $value)
		{
			JHtml::_('jquery.framework');
		}

	}

	/**
	 * Add normal stylesheet for the template
	 * @method addCSS
	 *
	 * @param  string  $url
	 * @param  boolean $useRoot
	 * @param  boolean $return
	 */
	public function addCSS($url, $useRoot = true, $return = true)
	{
		static $added = array();

		$url = $useRoot ? $this->root . '/templates/' . $this->document->template . '/css/' . $url : $url;

		if (!in_array($url, $added))
		{
			$added[] = $url;

			if ($return)
			{
				$this->document->addStyleSheet($url);
			}
			else
			{
				echo "\t" . '<link rel="stylesheet" href="' . $url . '" type="text/css" />' . "\r\n";
			}
		}

	}

	/**
	 * Add the main CSS file
	 * @method addTemplateCSS
	 */
	public function addTemplateCSS()
	{
	    $template_style_sheet = (file_exists(RSTEMPLATE_PATH . '/css/themes/template-' . $this->getTemplateId() . '.css')) ? 'themes/template-' . $this->getTemplateId() . '.css' : 'template.css' ;

        $this->document->addCustomTag('<link href="'.  $this->root . '/templates/' . $this->document->template . '/css/' . $template_style_sheet . '" rel="stylesheet" type="text/css" />');
	}

	/**
	 * Adds a custom stylesheet for the current component (com_content.css for example)
	 * @method addComponentCSS
	 */
	public function addComponentCSS()
	{
		$cssName = $this->option . '.css';
		$cssPath = RSTEMPLATE_PATH . '/css/components/' . $cssName;

		if (file_exists($cssPath))
		{
			$this->addCSS('components/' . $cssName, true, false);
		}

	}

	/**
	 * Adds a custom stylesheet for the modules if exist (mod_login.css for example)
	 * @method addModulesCSS
	 */
	public function addModulesCSS()
	{
		$xml       = simplexml_load_file(JPATH_SITE . "/templates/" . $this->document->template . "/templateDetails.xml");
		$positions = $xml->positions->position;

		if (count($positions))
		{
			foreach ($positions as $position)
			{
				$modules = JModuleHelper::getModules($position);
				foreach ($modules as $module)
				{
					$cssName = $module->module . '.css';
					$cssPath = RSTEMPLATE_PATH . '/css/modules/' . $cssName;
					if (file_exists($cssPath))
					{
						$this->addCSS('modules/' . $cssName, true, false);
					}
				}
			}
		}

	}

	/**
	 * Adds a custom.css file if exists
	 * @method addCustomCSS
	 */
	public function addCustomCSS()
	{
		$this->customCSSPath = RSTEMPLATE_PATH . '/css/custom.css';

		if (file_exists($this->customCSSPath))
		{
            $this->document->addCustomTag('<link href="'.  $this->root . '/templates/' . $this->document->template . '/css/custom.css" rel="stylesheet" type="text/css" />');
		}

	}

	/**
	 * Adds a custom.js file if exists
	 * @method addCustomJS
	 */
	public function addCustomJS()
	{
		$jsName = 'custom.js';

		if (file_exists(RSTEMPLATE_PATH . '/js/' . $jsName))
		{
			$this->addJS($jsName);
		}

	}

	/**
	 * Load the Bootstrap framework
	 * @method addBootstrap
	 */
	public function addBootstrap()
	{
		// Add JavaScript Frameworks
		JHtml::_('bootstrap.framework');
		// Load optional rtl Bootstrap css and Bootstrap bugfixes
		JHtmlBootstrap::loadCss($includeMaincss = true, $this->document->direction);
	}

	/**
	 * Get CSS path
	 * @method getCSSPath
	 *
	 * @param  string $category
	 * @param  string $file
	 *
	 * @return string
	 */
	public function getCSSPath($category, $file)
	{
		switch ($category)
		{
			case 'bootstrap':
				return '/media/jui/css/' . $file;
				break;
		}

		return false;

	}

	/**
	 * Get JS path
	 * @method getJSPath
	 *
	 * @param  string $category
	 * @param  string $file
	 *
	 * @return string
	 */
	public function getJSPath($category, $file)
	{
		switch ($category)
		{
			case 'bootstrap':
			case 'jquery':
				return '/media/jui/js/' . $file;
				break;
			case 'html5':
				return '/media/jui/js/' . $file;
				break;
		}

		return true;
	}

}
