<?php
/**
 * @version       1.0.0
 * @package       RSTrone! 1.0.0
 * @copyright (C) 2018 www.rsjoomla.com
 * @license       GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

defined('_JEXEC') or die;

/**
 * Render Widgets in the frontend
 */
class RSTemplateWidget
{
	/**
	 * @var string
	 */
	protected $name;
	/**
	 * @var string
	 */
	protected $path;
	/**
	 * @var object
	 */
	protected $params;

	/**
	 * Creates widgets from the template options
	 * @method __construct
	 *
	 * @param  string $widgetName
	 * @param  object $templateParams
	 */
	public function __construct($widgetName, $templateParams)
	{
		$this->name   = $widgetName;
		$this->path   = RSTEMPLATE_PATH . '/widgets/' . $widgetName;
		$this->params = $templateParams;
	}

	/**
	 * Function to use in the frontend $this->render('twitter')
	 * @method render
	 *
	 * @return string
	 */
	public function render()
	{
		$entryPoint = $this->path . '/' . $this->name . '.php';

		if (file_exists($entryPoint))
		{
			include $entryPoint;
		}

	}

	/**
	 * grabs the layout of the widget
	 * @method getLayout
	 *
	 * @param  string $layout
	 *
	 * @return string
	 */
	protected function getLayout($layout)
	{
		$layoutPath = $this->path . '/tmpl/' . $layout . '.php';

		if (file_exists($layoutPath))
		{
			return $layoutPath;
		}

		return false;
	}

	/**
	 * Escaping function
	 * @method escape
	 *
	 * @param  string $string
	 *
	 * @return string
	 */
	protected function escape($string)
	{
		return htmlentities($string, ENT_COMPAT, 'utf-8');
	}

}
