jQuery.fn.player = function() {
	var player_clone		= jQuery(this),
		player_container	= jQuery(this).parent(),
		current_src			= player_container.find('source').attr('src');
		
	player_container.append('<div class="rstpl-player"></div>');
	player_container = jQuery(this).next('.rstpl-player');
	
	jQuery(this).remove();
	
	if (jQuery(this).prop('nodeName') == 'AUDIO') {
		player_container.append('<div class="audio-container"></div>');
	} else {
		player_container.append('<div class="video-container"><span class="fa fa-play"></span></div>');
	}
	
	player_container.children().first().append(player_clone);
	player_container.append('<div class="controls"></div>');
	
	controls_container = player_container.find('.controls');
	
	// Add play / pause button
	addPlayPause(controls_container);
	
	// Add seek bar
	addSeekBar(controls_container);
	
	// Add volume bar
	addVolumeBar(controls_container);
	
	// Add title
	player_container.append('<div class="title">' + current_src.substr(current_src.lastIndexOf('/') + 1).replace(/\.[^/.]+$/, '') + '</div>');
	
	function addPlayPause(container) {
		var current_player = document.getElementById(container.parent().find('audio, video').attr('id'));
		
		container.append('<div class="play-pause"><span class="fa fa-play"></span></div>');
		
		container.find('.play-pause span').on('click', function() {
			if (current_player.paused) {
				current_player.play();
				jQuery(this).attr('class', 'fa fa-pause');
				container.parent().find('.video-container span').attr('class', 'fa fa-pause');
				container.parent().find('.video-container').addClass('playing');
			} else {
				current_player.pause();
				jQuery(this).attr('class', 'fa fa-play');
				container.parent().find('.video-container span').attr('class', 'fa fa-play');
				container.parent().find('.video-container').removeClass('playing');
			}
		});
		
		container.parent().find('.video-container').on('click', function() {
			if (current_player.paused) {
				current_player.play();
				jQuery(this).find('span').attr('class', 'fa fa-pause');
				jQuery(this).parent().find('.controls .play-pause span').attr('class', 'fa fa-pause');
				jQuery(this).addClass('playing');
			} else {
				current_player.pause();
				jQuery(this).find('span').attr('class', 'fa fa-play');
				jQuery(this).parent().find('.controls .play-pause span').attr('class', 'fa fa-play');
				jQuery(this).removeClass('playing');
			}
		});
	}

	function addSeekBar(container) {
		var current_player = document.getElementById(container.parent().find('audio, video').attr('id'));
		
		container.append('<div class="seek-container"><div class="seek-bar"><div class="seek-fill"></div></div><span class="timer">0:00</span></div>');
		
		container.find('.seek-bar').on('click', function(e) {
			var event_obj		= window.event ? event : e,
				click_offset	= event_obj.clientX - jQuery(this).offset().left,
				percentage		= click_offset / jQuery(this).width();
				
			current_player.currentTime = current_player.duration * percentage;
			
			jQuery(this).find('.seek-fill').css('width', Math.round(percentage * 100) + '%');
		});
		
		current_player.ontimeupdate = function() {
			var current_seconds	= (Math.floor(current_player.currentTime % 60) < 10 ? '0' : '') + Math.floor(current_player.currentTime % 60),
				current_minutes	= Math.floor(current_player.currentTime / 60);
				percentage		= current_player.currentTime / current_player.duration;
			
			// Update timer
			container.find('.timer').empty();
			container.find('.timer').append(current_minutes + ':' + current_seconds);
			
			// Update seek bar
			container.find('.seek-fill').css('width', Math.round(percentage * 100) + '%');
		};
	}

	function addVolumeBar(container) {
		var current_player		= document.getElementById(container.parent().find('audio, video').attr('id')),
			initial_percentage	= 1;
			
		container.append('<div class="volume-container"><span class="fa fa-volume-up"></span><div class="volume-bar"><div class="volume-fill"></div></div></div>');
		
		container.find('.volume-bar').on('click', function(e) {
			var event_obj		= window.event ? event : e,
				click_offset	= event_obj.clientX - jQuery(this).offset().left,
				percentage		= click_offset / jQuery(this).width();
				
			current_player.volume	= percentage;
			initial_percentage		= percentage;
			
			jQuery(this).find('.volume-fill').css('width', Math.round(percentage * 100) + '%');
			
			if (Math.round(percentage * 100) == 0) {
				jQuery(this).parent().find('span').attr('class', 'fa fa-volume-off');
			} else if (Math.round(percentage * 100) < 50) {
				jQuery(this).parent().find('span').attr('class', 'fa fa-volume-down');
			} else {
				jQuery(this).parent().find('span').attr('class', 'fa fa-volume-up');
			}
		});
		
		container.find('.volume-container span').on('click', function(e) {
			if (jQuery(this).attr('class') == 'fa fa-volume-off') {
				current_player.volume = initial_percentage;
				
				if (Math.round(initial_percentage * 100) > 50) {
					jQuery(this).attr('class', 'fa fa-volume-up');
				} else {
					jQuery(this).attr('class', 'fa fa-volume-down');
				}
				
				jQuery(this).parent().find('.volume-fill').css('width', Math.round(initial_percentage * 100) + '%');
			} else {
				current_player.volume = 0;
				jQuery(this).attr('class', 'fa fa-volume-off');
				jQuery(this).parent().find('.volume-fill').css('width', '0');
			}
		});
	}
}
